# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2024/5/19 18:34
Desc: 巨潮资讯-行业分类数据
https://webapi.cninfo.com.cn/#/apiDoc
https://webapi.cninfo.com.cn/api/stock/p_stock2110
"""

import numpy as np
import pandas as pd
import requests
import py_mini_racer

from byteapp_ak.datasets import get_ths_js


def _get_file_content_ths(file: str = "cninfo.js") -> str:
    """
    获取 JS 文件的内容
    :param file:  JS 文件名
    :type file: str
    :return: 文件内容
    :rtype: str
    """
    setting_file_path = get_ths_js(file)
    with open(setting_file_path, encoding="utf-8") as f:
        file_data = f.read()
    return file_data


def stock_industry_category_cninfo(symbol: str = "巨潮行业分类标准") -> pd.DataFrame:
    """
    巨潮资讯-行业分类数据
    https://webapi.cninfo.com.cn/#/apiDoc
    查询 p_public0002 接口
    :param symbol: 行业类型; choice of {"证监会行业分类标准", "巨潮行业分类标准", "申银万国行业分类标准",
    "新财富行业分类标准", "国资委行业分类标准", "巨潮产业细分标准", "天相行业分类标准", "全球行业分类标准"}
    :type symbol: str
    :return: 行业分类数据
    :rtype: pandas.DataFrame
    """
    symbol_map = {
        "证监会行业分类标准": "008001",
        "巨潮行业分类标准": "008002",
        "申银万国行业分类标准": "008003",
        "新财富行业分类标准": "008004",
        "国资委行业分类标准": "008005",
        "巨潮产业细分标准": "008006",
        "天相行业分类标准": "008007",
        "全球行业分类标准": "008008",
    }
    url = "https://webapi.cninfo.com.cn/api/stock/p_public0002"
    params = {"indcode": "", "indtype": symbol_map[symbol], "format": "json"}
    js_code = py_mini_racer.MiniRacer()
    js_content = _get_file_content_ths("cninfo.js")
    js_code.eval(js_content)
    mcode = js_code.call("getResCode1")
    headers = {
        "Accept": "*/*",
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Content-Length": "0",
        "Host": "webapi.cninfo.com.cn",
        "Accept-Enckey": mcode,
        "Origin": "https://webapi.cninfo.com.cn",
        "Pragma": "no-cache",
        "Proxy-Connection": "keep-alive",
        "Referer": "https://webapi.cninfo.com.cn/",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/93.0.4577.63 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["records"])
    cols_map = {
        "PARENTCODE": "父类编码",
        "SORTCODE": "类目编码",
        "SORTNAME": "类目名称",
        "F001V": "类目名称英文",
        "F002D": "终止日期",
        "F003V": "行业类型编码",
        "F004V": "行业类型",
    }
    temp_df.rename(columns=cols_map, inplace=True)
    # 行业按分级排序
    tmp = temp_df[["类目编码"]].copy()
    tmp["len"] = temp_df["类目编码"].str.len()
    tmp["Level"] = 0
    g = tmp.groupby("len")
    level = 0
    for k in g.groups.keys():
        temp_df.loc[temp_df["类目编码"].isin(g.get_group(k)["类目编码"]), "Level"] = (
            level
        )
        level += 1
    temp_df["Level"] = temp_df["Level"].astype(int)
    temp_df.rename(columns={"Level": "分级"}, inplace=True)
    temp_df["终止日期"] = pd.to_datetime(temp_df["终止日期"], errors="coerce").dt.date
    return temp_df


def stock_industry_change_cninfo(
    symbol: str = "002594",
    start_date: str = "20091227",
    end_date: str = "20220713",
) -> pd.DataFrame:
    """
    巨潮资讯-上市公司行业归属的变动情况
    https://webapi.cninfo.com.cn/#/apiDoc
    查询 p_stock2110 接口
    :param symbol: 股票代码
    :type symbol: str
    :param start_date: 开始变动日期
    :type start_date: str
    :param end_date: 结束变动日期
    :type end_date: str
    :return: 行业归属的变动情况
    :rtype: pandas.DataFrame
    """
    url = "https://webapi.cninfo.com.cn/api/stock/p_stock2110"
    params = {
        "scode": symbol,
        "sdate": "-".join([start_date[:4], start_date[4:6], start_date[6:]]),
        "edate": "-".join([end_date[:4], end_date[4:6], end_date[6:]]),
    }

    js_code = py_mini_racer.MiniRacer()
    js_content = _get_file_content_ths("cninfo.js")
    js_code.eval(js_content)
    mcode = js_code.call("getResCode1")
    headers = {
        "Accept": "*/*",
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Content-Length": "0",
        "Host": "webapi.cninfo.com.cn",
        "Accept-Enckey": mcode,
        "Origin": "https://webapi.cninfo.com.cn",
        "Pragma": "no-cache",
        "Proxy-Connection": "keep-alive",
        "Referer": "https://webapi.cninfo.com.cn/",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/93.0.4577.63 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    r = requests.post(url, params=params, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["records"])
    cols_map = {
        "ORGNAME": "机构名称",
        "SECCODE": "证券代码",
        "SECNAME": "新证券简称",
        "VARYDATE": "变更日期",
        "F001V": "分类标准编码",
        "F002V": "分类标准",
        "F003V": "行业编码",
        "F004V": "行业门类",
        "F005V": "行业次类",
        "F006V": "行业大类",
        "F007V": "行业中类",
        "F008C": "最新记录标识",
    }
    ignore_cols = ["最新记录标识"]
    temp_df.rename(columns=cols_map, inplace=True)
    temp_df.fillna(np.nan, inplace=True)
    temp_df["变更日期"] = pd.to_datetime(temp_df["变更日期"]).dt.date
    data_df = temp_df[[c for c in temp_df.columns if c not in ignore_cols]]
    return data_df


if __name__ == "__main__":
    stock_industry_category_cninfo_df = stock_industry_category_cninfo(
        symbol="巨潮行业分类标准"
    )
    print(stock_industry_category_cninfo_df)

    stock_industry_change_cninfo_df = stock_industry_change_cninfo(
        symbol="002594", start_date="20091227", end_date="20220708"
    )
    print(stock_industry_change_cninfo_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2022/10/10 17:26
Desc: 百度股市通- A 股或指数-股评-投票
https://gushitong.baidu.com/index/ab-000001
"""
import requests
import pandas as pd


def stock_zh_vote_baidu(symbol: str = "000001", indicator: str = "指数") -> pd.DataFrame:
    """
    百度股市通- A 股或指数-股评-投票
    https://gushitong.baidu.com/index/ab-000001
    :param symbol: 股票代码
    :type symbol: str
    :param indicator: choice of {"指数", "股票"}
    :type indicator: str
    :return: 投票数据
    :rtype: pandas.DataFrame
    """
    indicator_map = {"股票": "stock", "指数": "index"}
    url = "https://finance.pae.baidu.com/vapi/v1/stockvoterecords"
    params = {
        "code": symbol,
        "market": "ab",
        "finance_type": indicator_map[indicator],
        "select_type": "week",
        "from_smart_app": "0",
        "method": "query",
        "finClientType": "pc",
    }
    temp_list = []
    for item_period in ["day", "week", "month", "year"]:
        params.update({"select_type": item_period})
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_list.append(
            [
                item
                for item in data_json["Result"]["voteRecords"]["voteRes"]
                if item["type"] == item_period
            ][0]
        )
    temp_df = pd.DataFrame(temp_list)
    temp_df.columns = ["周期", "-", "看涨", "看跌", "看涨比例", "看跌比例"]
    temp_df = temp_df[["周期", "看涨", "看跌", "看涨比例", "看跌比例"]]
    return temp_df


if __name__ == "__main__":
    stock_zh_vote_baidu_df = stock_zh_vote_baidu(symbol="000001", indicator="指数")
    print(stock_zh_vote_baidu_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/10/18 15:00
Desc: 全部A股-等权重市盈率、中位数市盈率
https://www.legulegu.com/stockdata/a-ttm-lyr
"""

import pandas as pd
import requests

from byteapp_ak.stock_feature.stock_a_indicator import get_token_lg, get_cookie_csrf


def stock_a_ttm_lyr() -> pd.DataFrame:
    """
    全部 A 股-等权重市盈率、中位数市盈率
    https://www.legulegu.com/stockdata/a-ttm-lyr
    :return: 全部A股-等权重市盈率、中位数市盈率
    :rtype: pandas.DataFrame
    """
    url = "https://legulegu.com/api/stock-data/market-ttm-lyr"
    params = {
        "marketId": "5",
        "token": get_token_lg(),
    }
    r = requests.get(
        url,
        params=params,
        **get_cookie_csrf(url="https://www.legulegu.com/stockdata/a-ttm-lyr"),
    )
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"])
    temp_df["date"] = pd.to_datetime(temp_df["date"], errors="coerce").dt.date
    return temp_df


if __name__ == "__main__":
    stock_a_ttm_lyr_df = stock_a_ttm_lyr()
    print(stock_a_ttm_lyr_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2022/11/12 17:14
Desc: 东方财富-经济数据-英国
https://data.eastmoney.com/cjsj/foreign_4_0.html
"""
import pandas as pd
import requests


def macro_uk_core(symbol: str = "EMG00010348") -> pd.DataFrame:
    """
    东方财富-数据中心-经济数据一览-宏观经济-英国-核心代码
    https://data.eastmoney.com/cjsj/foreign_4_0.html
    :param symbol: 代码
    :type symbol: str
    :return: 指定 symbol 的数据
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "reportName": "RPT_ECONOMICVALUE_BRITAIN",
        "columns": "ALL",
        "filter": f'(INDICATOR_ID="{symbol}")',
        "pageNumber": "1",
        "pageSize": "5000",
        "sortColumns": "REPORT_DATE",
        "sortTypes": "-1",
        "source": "WEB",
        "client": "WEB",
        "p": "1",
        "pageNo": "1",
        "pageNum": "1",
        "_": "1667639896816",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_df.rename(
        columns={
            "COUNTRY": "-",
            "INDICATOR_ID": "-",
            "INDICATOR_NAME": "-",
            "REPORT_DATE_CH": "时间",
            "REPORT_DATE": "-",
            "PUBLISH_DATE": "发布日期",
            "VALUE": "现值",
            "PRE_VALUE": "前值",
            "INDICATOR_IDOLD": "-",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "时间",
            "前值",
            "现值",
            "发布日期",
        ]
    ]
    temp_df["前值"] = pd.to_numeric(temp_df["前值"])
    temp_df["现值"] = pd.to_numeric(temp_df["现值"])
    temp_df["发布日期"] = pd.to_datetime(temp_df["发布日期"]).dt.date
    temp_df.sort_values(["发布日期"], inplace=True, ignore_index=True)
    return temp_df


# Halifax房价指数月率
def macro_uk_halifax_monthly() -> pd.DataFrame:
    """
    东方财富-经济数据-英国-Halifax 房价指数月率
    https://data.eastmoney.com/cjsj/foreign_4_0.html
    :return: Halifax 房价指数月率
    :rtype: pandas.DataFrame
    """
    temp_df = macro_uk_core(symbol="EMG00342256")
    return temp_df


# Halifax 房价指数年率
def macro_uk_halifax_yearly() -> pd.DataFrame:
    """
    东方财富-经济数据-英国-Halifax 房价指数年率
    https://data.eastmoney.com/cjsj/foreign_4_1.html
    :return: Halifax房价指数年率
    :rtype: pandas.DataFrame
    """
    temp_df = macro_uk_core(symbol="EMG00010370")
    return temp_df


# 贸易帐
def macro_uk_trade() -> pd.DataFrame:
    """
    东方财富-经济数据-英国-贸易帐
    https://data.eastmoney.com/cjsj/foreign_4_2.html
    :return: 贸易帐
    :rtype: pandas.DataFrame
    """
    temp_df = macro_uk_core(symbol="EMG00158309")
    return temp_df


# 央行公布利率决议
def macro_uk_bank_rate() -> pd.DataFrame:
    """
    东方财富-经济数据-英国-央行公布利率决议
    https://data.eastmoney.com/cjsj/foreign_4_3.html
    :return: 央行公布利率决议
    :rtype: pandas.DataFrame
    """
    temp_df = macro_uk_core(symbol="EMG00342253")
    return temp_df


# 核心消费者物价指数年率
def macro_uk_core_cpi_yearly() -> pd.DataFrame:
    """
    东方财富-经济数据-英国-核心消费者物价指数年率
    https://data.eastmoney.com/cjsj/foreign_4_4.html
    :return: 核心消费者物价指数年率
    :rtype: pandas.DataFrame
    """
    temp_df = macro_uk_core(symbol="EMG00010279")
    return temp_df


# 核心消费者物价指数月率
def macro_uk_core_cpi_monthly() -> pd.DataFrame:
    """
    东方财富-经济数据-英国-核心消费者物价指数月率
    https://data.eastmoney.com/cjsj/foreign_4_5.html
    :return: 核心消费者物价指数月率
    :rtype: pandas.DataFrame
    """
    temp_df = macro_uk_core(symbol="EMG00010291")
    return temp_df


# 消费者物价指数年率
def macro_uk_cpi_yearly() -> pd.DataFrame:
    """
    东方财富-经济数据-英国-消费者物价指数年率
    https://data.eastmoney.com/cjsj/foreign_4_6.html
    :return: 消费者物价指数年率
    :rtype: pandas.DataFrame
    """
    temp_df = macro_uk_core(symbol="EMG00010267")
    return temp_df


# 消费者物价指数月率
def macro_uk_cpi_monthly() -> pd.DataFrame:
    """
    东方财富-经济数据-英国-消费者物价指数月率
    https://data.eastmoney.com/cjsj/foreign_4_7.html
    :return: 消费者物价指数月率
    :rtype: pandas.DataFrame
    """
    temp_df = macro_uk_core(symbol="EMG00010291")
    return temp_df


# 零售销售月率
def macro_uk_retail_monthly() -> pd.DataFrame:
    """
    东方财富-经济数据-英国-零售销售月率
    https://data.eastmoney.com/cjsj/foreign_4_8.html
    :return: 零售销售月率
    :rtype: pandas.DataFrame
    """
    temp_df = macro_uk_core(symbol="EMG00158298")
    return temp_df


# 零售销售年率
def macro_uk_retail_yearly() -> pd.DataFrame:
    """
    东方财富-经济数据-英国-零售销售年率
    https://data.eastmoney.com/cjsj/foreign_4_9.html
    :return: 零售销售年率
    :rtype: pandas.DataFrame
    """
    temp_df = macro_uk_core(symbol="EMG00158297")
    return temp_df


# Rightmove 房价指数年率
def macro_uk_rightmove_yearly() -> pd.DataFrame:
    """
    东方财富-经济数据-英国-Rightmove 房价指数年率
    https://data.eastmoney.com/cjsj/foreign_4_10.html
    :return: Rightmove 房价指数年率
    :rtype: pandas.DataFrame
    """
    temp_df = macro_uk_core(symbol="EMG00341608")
    return temp_df


# Rightmove 房价指数月率
def macro_uk_rightmove_monthly() -> pd.DataFrame:
    """
    东方财富-经济数据-英国-Rightmove 房价指数月率
    https://data.eastmoney.com/cjsj/foreign_4_11.html
    :return: Rightmove 房价指数月率
    :rtype: pandas.DataFrame
    """
    temp_df = macro_uk_core(symbol="EMG00341607")
    return temp_df


# GDP 季率初值
def macro_uk_gdp_quarterly() -> pd.DataFrame:
    """
    东方财富-经济数据-英国-GDP 季率初值
    https://data.eastmoney.com/cjsj/foreign_4_12.html
    :return: GDP 季率初值
    :rtype: pandas.DataFrame
    """
    temp_df = macro_uk_core(symbol="EMG00158277")
    return temp_df


# GDP 年率初值
def macro_uk_gdp_yearly() -> pd.DataFrame:
    """
    东方财富-经济数据-英国-GDP 年率初值
    https://data.eastmoney.com/cjsj/foreign_4_13.html
    :return: GDP 年率初值
    :rtype: pandas.DataFrame
    """
    temp_df = macro_uk_core(symbol="EMG00158276")
    return temp_df


# 失业率
def macro_uk_unemployment_rate() -> pd.DataFrame:
    """
    东方财富-经济数据-英国-失业率
    https://data.eastmoney.com/cjsj/foreign_4_14.html
    :return: 失业率
    :rtype: pandas.DataFrame
    """
    temp_df = macro_uk_core(symbol="EMG00010348")
    return temp_df


if __name__ == "__main__":
    macro_uk_halifax_monthly_df = macro_uk_halifax_monthly()
    print(macro_uk_halifax_monthly_df)

    macro_uk_halifax_yearly_df = macro_uk_halifax_yearly()
    print(macro_uk_halifax_yearly_df)

    macro_uk_trade_df = macro_uk_trade()
    print(macro_uk_trade_df)

    macro_uk_bank_rate_df = macro_uk_bank_rate()
    print(macro_uk_bank_rate_df)

    macro_uk_core_cpi_yearly_df = macro_uk_core_cpi_yearly()
    print(macro_uk_core_cpi_yearly_df)

    macro_uk_core_cpi_monthly_df = macro_uk_core_cpi_monthly()
    print(macro_uk_core_cpi_monthly_df)

    macro_uk_cpi_yearly_df = macro_uk_cpi_yearly()
    print(macro_uk_cpi_yearly_df)

    macro_uk_cpi_monthly_df = macro_uk_cpi_monthly()
    print(macro_uk_cpi_monthly_df)

    macro_uk_retail_monthly_df = macro_uk_retail_monthly()
    print(macro_uk_retail_monthly_df)

    macro_uk_retail_yearly_df = macro_uk_retail_yearly()
    print(macro_uk_retail_yearly_df)

    macro_uk_rightmove_yearly_df = macro_uk_rightmove_yearly()
    print(macro_uk_rightmove_yearly_df)

    macro_uk_rightmove_monthly_df = macro_uk_rightmove_monthly()
    print(macro_uk_rightmove_monthly_df)

    macro_uk_gdp_quarterly_df = macro_uk_gdp_quarterly()
    print(macro_uk_gdp_quarterly_df)

    macro_uk_gdp_yearly_df = macro_uk_gdp_yearly()
    print(macro_uk_gdp_yearly_df)

    macro_uk_unemployment_rate_df = macro_uk_unemployment_rate()
    print(macro_uk_unemployment_rate_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/5/12 22:30
Desc: 百度地图慧眼-百度迁徙数据
"""

import json

import pandas as pd
import requests

from byteapp_ak.event.cons import province_dict, city_dict


def migration_area_baidu(
    area: str = "重庆市", indicator: str = "move_in", date: str = "20230922"
) -> pd.DataFrame:
    """
    百度地图慧眼-百度迁徙-XXX迁入地详情
    百度地图慧眼-百度迁徙-XXX迁出地详情
    以上展示 top100 结果，如不够 100 则展示全部
    迁入来源地比例: 从 xx 地迁入到当前区域的人数与当前区域迁入总人口的比值
    迁出目的地比例: 从当前区域迁出到 xx 的人口与从当前区域迁出总人口的比值
    https://qianxi.baidu.com/?from=shoubai#city=0
    :param area: 可以输入 省份 或者 具体城市 但是需要用全称
    :type area: str
    :param indicator: move_in 迁入 move_out 迁出
    :type indicator: str
    :param date: 查询的日期 20200101 以后的时间
    :type date: str
    :return: 迁入地详情/迁出地详情的前 50 个
    :rtype: pandas.DataFrame
    """
    city_dict.update(province_dict)
    inner_dict = dict(zip(city_dict.values(), city_dict.keys()))
    if inner_dict[area] in province_dict.keys():
        dt_flag = "province"
    else:
        dt_flag = "city"
    url = "https://huiyan.baidu.com/migration/cityrank.jsonp"
    params = {
        "dt": dt_flag,
        "id": inner_dict[area],
        "type": indicator,
        "date": date,
    }
    r = requests.get(url, params=params)
    data_text = r.text[r.text.find("({") + 1 : r.text.rfind(");")]
    data_json = json.loads(data_text)
    temp_df = pd.DataFrame(data_json["data"]["list"])
    temp_df["value"] = pd.to_numeric(temp_df["value"], errors="coerce")
    return temp_df


def migration_scale_baidu(
    area: str = "广州市",
    indicator: str = "move_in",
) -> pd.DataFrame:
    """
    百度地图慧眼-百度迁徙-迁徙规模
    迁徙规模指数：反映迁入或迁出人口规模，城市间可横向对比城市迁徙边界采用该城市行政区划，包含该城市管辖的区、县、乡、村
    https://qianxi.baidu.com/?from=shoubai#city=0
    :param area: 可以输入 省份 或者 具体城市 但是需要用全称
    :type area: str
    :param indicator: move_in 迁入 move_out 迁出
    :type indicator: str
    :return: 时间序列的迁徙规模指数
    :rtype: pandas.DataFrame
    """
    city_dict.update(province_dict)
    inner_dict = dict(zip(city_dict.values(), city_dict.keys()))
    if inner_dict[area] in province_dict.keys():
        dt_flag = "province"
    else:
        dt_flag = "city"
    url = "https://huiyan.baidu.com/migration/historycurve.jsonp"
    params = {
        "dt": dt_flag,
        "id": inner_dict[area],
        "type": indicator,
    }
    r = requests.get(url, params=params)
    json_data = json.loads(r.text[r.text.find("({") + 1 : r.text.rfind(");")])
    temp_df = pd.DataFrame.from_dict(json_data["data"]["list"], orient="index")
    temp_df.index = pd.to_datetime(temp_df.index)
    temp_df.reset_index(inplace=True)
    temp_df.columns = ["日期", "迁徙规模指数"]
    temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date
    temp_df["迁徙规模指数"] = pd.to_numeric(temp_df["迁徙规模指数"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    migration_area_baidu_df = migration_area_baidu(
        area="杭州市", indicator="move_out", date="20240401"
    )
    print(migration_area_baidu_df)

    migration_scale_baidu_df = migration_scale_baidu(
        area="广州市",
        indicator="move_in",
    )
    print(migration_scale_baidu_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2022/3/5 17:00
Desc: 金十数据中心-实时监控-微博舆情报告
https://datacenter.jin10.com/market
报告内容：时间期限可选择2小时、6小时、12小时、1天、1周、1月。
该表格展示的是在对应的时间期限内，个股在微博讨论中的人气排行指数。
红色颜色越深，表明该股票讨论热度越高，其当前的涨幅更大。
绿色颜色越深，表明该股票讨论的热度越低，其当前的跌幅更大。
"""
import time
from typing import Dict

import pandas as pd
import requests


def stock_js_weibo_nlp_time() -> Dict:
    """
    https://datacenter.jin10.com/market
    :return: 特定时间表示的字典
    :rtype: dict
    """
    url = "https://datacenter-api.jin10.com/weibo/config"
    payload = {"_": int(time.time() * 1000)}
    headers = {
        "authority": "datacenter-api.jin10.com",
        "pragma": "no-cache",
        "cache-control": "no-cache",
        "accept": "*/*",
        "x-app-id": "rU6QIu7JHe2gOUeR",
        "sec-fetch-dest": "empty",
        "x-csrf-token": "",
        "x-version": "1.0.0",
        "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/80.0.3987.116 Safari/537.36",
        "origin": "https://datacenter.jin10.com",
        "sec-fetch-site": "same-site",
        "sec-fetch-mode": "cors",
        "referer": "https://datacenter.jin10.com/market",
        "accept-language": "zh-CN,zh;q=0.9,en;q=0.8",
    }

    r = requests.get(url, headers=headers, data=payload)
    return r.json()["data"]["timescale"]


def stock_js_weibo_report(time_period: str = "CNHOUR12") -> pd.DataFrame:
    """
    金十数据中心-实时监控-微博舆情报告
    https://datacenter.jin10.com/market
    :param time_period: {'CNHOUR2': '2小时', 'CNHOUR6': '6小时', 'CNHOUR12': '12小时', 'CNHOUR24': '1天', 'CNDAY7': '1周', 'CNDAY30': '1月'}
    :type time_period: str
    :return: 指定时间段的微博舆情报告
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-api.jin10.com/weibo/list"
    payload = {
        "timescale": time_period,
        "_": int(time.time() * 1000)
    }
    headers = {
        'authority': 'datacenter-api.jin10.com',
        'pragma': 'no-cache',
        'cache-control': 'no-cache',
        'accept': '*/*',
        'x-app-id': 'rU6QIu7JHe2gOUeR',
        'sec-fetch-dest': 'empty',
        'x-csrf-token': '',
        'x-version': '1.0.0',
        'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/80.0.3987.116 Safari/537.36',
        'origin': 'https://datacenter.jin10.com',
        'sec-fetch-site': 'same-site',
        'sec-fetch-mode': 'cors',
        'referer': 'https://datacenter.jin10.com/market',
        'accept-language': 'zh-CN,zh;q=0.9,en;q=0.8'
    }

    r = requests.get(url, params=payload, headers=headers)
    temp_df = pd.DataFrame(r.json()["data"])
    temp_df['rate'] = pd.to_numeric(temp_df['rate'])
    return temp_df


if __name__ == '__main__':
    stock_js_weibo_nlp_time_map = stock_js_weibo_nlp_time()
    print(stock_js_weibo_nlp_time_map)

    get_news_df = stock_js_weibo_report(time_period="CNHOUR6")
    print(get_news_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2023/3/5 18:12
Desc: 百度股市通-外汇-行情榜单
https://gushitong.baidu.com/top/foreign-common-%E5%B8%B8%E7%94%A8
"""
import http.client
import json
import urllib

import pandas as pd


def fx_quote_baidu(symbol: str = "人民币") -> pd.DataFrame:
    """
    百度股市通-外汇-行情榜单
    https://gushitong.baidu.com/top/foreign-common-%E5%B8%B8%E7%94%A8
    :param symbol: choice of {"人民币", 美元"}
    :type symbol: str
    :return: 外汇行情数据
    :rtype: pandas.DataFrame
    """
    symbol_map = {
        "人民币": "rmb",
        "美元": "dollar",
    }
    num = 0
    out_df = pd.DataFrame()
    while True:
        try:
            conn = http.client.HTTPSConnection("finance.pae.baidu.com")
            params = {
                "pn": num,
                "rn": "20",
                "type": symbol_map[symbol],
                "finClientType": "pc",
            }
            conn.request("GET", f"/api/getforeignrank?{urllib.parse.urlencode(params)}")
            res = conn.getresponse()
            data = res.read()
            data_json = json.loads(data.decode("utf-8"))
            temp_df = pd.DataFrame(data_json["Result"])
            temp_list = []
            for item in temp_df["list"]:
                temp_list.append(list(pd.DataFrame(item).T.iloc[1, :].values))
            value_df = pd.DataFrame(
                temp_list, columns=pd.DataFrame(item).T.iloc[0, :].values
            )
            big_df = pd.concat([temp_df, value_df], axis=1)
            del big_df["market"]
            del big_df["list"]
            del big_df["status"]
            del big_df["icon1"]
            del big_df["icon2"]
            big_df.columns = ["代码", "名称", "最新价", "涨跌额", "涨跌幅"]
            big_df["最新价"] = pd.to_numeric(big_df["最新价"])
            big_df["涨跌额"] = pd.to_numeric(big_df["涨跌额"])
            big_df["涨跌幅"] = pd.to_numeric(big_df["涨跌幅"].str.strip("%")) / 100
            out_df = pd.concat([out_df, big_df], ignore_index=True)
            num = num + 20
        except:
            break
    return out_df


if __name__ == "__main__":
    fx_quote_baidu_df = fx_quote_baidu(symbol="人民币")
    print(fx_quote_baidu_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2023/7/2 16:00
Desc: 百度股市通-期货-新闻
https://gushitong.baidu.com/futures/ab-CJ888
"""
import pandas as pd
import requests


def futures_news_baidu(symbol: str = "AL") -> pd.DataFrame:
    """
    百度股市通-期货-新闻
    https://gushitong.baidu.com/futures/ab-CJ888
    :param symbol: 期货品种代码；大写
    :type symbol: str
    :return: 新闻
    :rtype: pandas.DataFrame
    """
    url = "https://finance.pae.baidu.com/vapi/getfuturesnews"
    params = {"code": f"{symbol}888", "pn": "0", "rn": "2000", "finClientType": "pc"}
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["Result"])
    temp_df.rename(
        columns={
            "loc": "-",
            "provider": "-",
            "source": "-",
            "publish_time": "发布时间",
            "third_url": "新闻链接",
            "title": "标题",
            "is_self_build": "-",
            "news_id": "-",
            "locate_url": "-",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "标题",
            "发布时间",
            "新闻链接",
        ]
    ]
    temp_df["发布时间"] = pd.to_datetime(pd.to_numeric(temp_df["发布时间"]), unit="s").dt.date
    temp_df.sort_values(["发布时间"], inplace=True, ignore_index=True)
    return temp_df


if __name__ == "__main__":
    futures_news_baidu_df = futures_news_baidu(symbol="AL")
    print(futures_news_baidu_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2023/4/3 21:06
Desc: 银保监会配置文件
"""
cbirc_headers_without_cookie_2020 = {
    "Accept": "*/*",
    "Accept-Encoding": "gzip, deflate",
    "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
    "Cache-Control": "no-cache",
    "Connection": "keep-alive",
    "Host": "www.cbirc.gov.cn",
    "Pragma": "no-cache",
    "Referer": "http://www.cbirc.gov.cn/cn/view/pages/ItemList.html?itemPId=923&itemId=4115&itemUrl=ItemListRightList.html&itemName=%E9%93%B6%E4%BF%9D%E7%9B%91%E5%88%86%E5%B1%80%E6%9C%AC%E7%BA%A7&itemsubPId=931&itemsubPName=%E8%A1%8C%E6%94%BF%E5%A4%84%E7%BD%9A",
    "X-Requested-With": "XMLHttpRequest",
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/78.0.3904.97 Safari/537.36"
}

cbirc_headers_without_cookie_2019 = {
    "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3",
    "Accept-Encoding": "gzip, deflate",
    "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
    "Cache-Control": "no-cache",
    "Connection": "keep-alive",
    "Host": "www.cbirc.gov.cn",
    "Pragma": "no-cache",
    "Referer": "http://www.cbirc.gov.cn/cn/list/9103/910305/ybjjcf/1.html",
    "Upgrade-Insecure-Requests": "1",
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/78.0.3904.97 Safari/537.36"
}



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/01/16 23:40
Desc: 雪球基金-基金详情
https://danjuanfunds.com/funding/003545
"""

import pandas as pd
import requests


def fund_individual_basic_info_xq(
    symbol: str = "000001", timeout: float = None
) -> pd.DataFrame:
    """
    雪球基金-基金详情
    https://danjuanfunds.com/djapi/fund/675091
    :param symbol: 基金代码
    :type symbol: str
    :param timeout: choice of None or a positive float number
    :type timeout: float
    :return: 基金信息
    :rtype: pandas.DataFrame
    """
    url = f"https://danjuanfunds.com/djapi/fund/{symbol}"
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/80.0.3987.149 Safari/537.36"
    }
    r = requests.get(url, headers=headers, timeout=timeout)
    json_data = r.json()["data"]
    temp_df = pd.json_normalize(json_data)
    temp_df.rename(
        columns={
            "fd_code": "基金代码",
            "fd_name": "基金名称",
            "fd_full_name": "基金全称",
            "found_date": "成立时间",
            "totshare": "最新规模",
            "keeper_name": "基金公司",
            "manager_name": "基金经理",
            "trup_name": "托管银行",
            "type_desc": "基金类型",
            "rating_source": "评级机构",
            "rating_desc": "基金评级",
            "invest_orientation": "投资策略",
            "invest_target": "投资目标",
            "performance_bench_mark": "业绩比较基准",
        },
        inplace=True,
    )
    if "评级机构" not in temp_df.columns:
        temp_df["评级机构"] = pd.NA
    temp_df = temp_df[
        [
            "基金代码",
            "基金名称",
            "基金全称",
            "成立时间",
            "最新规模",
            "基金公司",
            "基金经理",
            "托管银行",
            "基金类型",
            "评级机构",
            "基金评级",
            "投资策略",
            "投资目标",
            "业绩比较基准",
        ]
    ]
    temp_df = temp_df.T.reset_index()
    temp_df.columns = ["item", "value"]
    return temp_df


def fund_individual_achievement_xq(
    symbol: str = "000001", timeout: float = None
) -> pd.DataFrame:
    """
    雪球基金-基金业绩
    https://danjuanfunds.com/djapi/fundx/base/fund/achievement/675091
    :param symbol: 基金代码
    :type symbol: str
    :param timeout: choice of None or a positive float number
    :type timeout: float
    :return: 基金业绩
    :rtype: pandas.DataFrame
    """
    url = f"https://danjuanfunds.com/djapi/fundx/base/fund/achievement/{symbol}"
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/80.0.3987.149 Safari/537.36"
    }
    r = requests.get(url, headers=headers, timeout=timeout)
    json_data = r.json()["data"]
    combined_df = None
    type_dict = {
        "annual_performance_list": "年度业绩",
        "stage_performance_list": "阶段业绩",
    }
    for k, v in type_dict.items():
        temp_df = pd.DataFrame.from_dict(json_data[k], orient="columns")
        temp_df["type"] = v
        temp_df = temp_df[
            [
                "type",
                "period_time",
                "self_nav",
                "self_max_draw_down",
                "self_nav_rank",
            ]
        ]
        temp_df.columns = [
            "业绩类型",
            "周期",
            "本产品区间收益",
            "本产品最大回撒",
            "周期收益同类排名",
        ]
        combined_df = pd.concat([combined_df, temp_df], ignore_index=True)
    combined_df = combined_df.map(
        lambda x: x if "%" not in str(x) else x.replace("%", "")
    )
    combined_df[["本产品区间收益", "本产品最大回撒"]] = combined_df[
        ["本产品区间收益", "本产品最大回撒"]
    ].astype(float)
    return combined_df


def fund_individual_analysis_xq(
    symbol: str = "000001", timeout: float = None
) -> pd.DataFrame:
    """
    雪球基金-基金数据分析
    https://danjuanfunds.com/djapi/fund/base/quote/data/index/analysis/675091
    :param symbol: 基金代码
    :type symbol: str
    :param timeout: choice of None or a positive float number
    :type timeout: float
    :return: 基金数据分析
    :rtype: pandas.DataFrame
    """
    url = f"https://danjuanfunds.com/djapi/fund/base/quote/data/index/analysis/{symbol}"
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/80.0.3987.149 Safari/537.36"
    }
    r = requests.get(url, headers=headers, timeout=timeout)
    json_data = r.json()["data"]["index_data_list"]
    temp_df = pd.json_normalize(json_data)
    temp_df = temp_df[
        [
            "index_time_period",
            "investment_cost_performance",
            "risk_control",
            "self_index.volatility_rank",
            "self_index.sharpe_rank",
            "self_index.max_draw_down",
        ]
    ]
    temp_df.columns = [
        "周期",
        "较同类风险收益比",
        "较同类抗风险波动",
        "年化波动率",
        "年化夏普比率",
        "最大回撤",
    ]
    temp_df = temp_df.map(lambda x: x if "%" not in str(x) else x.replace("%", ""))
    temp_df[["年化波动率", "最大回撤"]] *= 100
    temp_df["较同类风险收益比"] = pd.to_numeric(
        temp_df["较同类风险收益比"], errors="coerce"
    )
    temp_df["较同类抗风险波动"] = pd.to_numeric(
        temp_df["较同类抗风险波动"], errors="coerce"
    )
    temp_df["年化波动率"] = pd.to_numeric(temp_df["年化波动率"], errors="coerce")
    temp_df["年化夏普比率"] = pd.to_numeric(temp_df["年化夏普比率"], errors="coerce")
    temp_df["最大回撤"] = pd.to_numeric(temp_df["最大回撤"], errors="coerce")
    return temp_df


def fund_individual_profit_probability_xq(
    symbol: str = "000001", timeout: float = None
) -> pd.DataFrame:
    """
    雪球基金-盈利概率-历史任意时点买入，持有满 X 年，盈利概率 Y%
    https://danjuanfunds.com/djapi/fundx/base/fund/profit/ratio/675091
    :param symbol: 基金代码
    :type symbol: str
    :param timeout: choice of None or a positive float number
    :type timeout: float
    :return: 盈利概率
    :rtype: pandas.DataFrame
    """
    url = f"https://danjuanfunds.com/djapi/fundx/base/fund/profit/ratio/{symbol}"
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/80.0.3987.149 Safari/537.36"
    }
    r = requests.get(url, headers=headers, timeout=timeout)
    json_data = r.json()["data"]["data_list"]
    temp_df = pd.DataFrame.from_dict(json_data, orient="columns")
    temp_df = temp_df[
        [
            "holding_time",
            "profit_ratio",
            "average_income",
        ]
    ]
    temp_df.columns = [
        "持有时长",
        "盈利概率",
        "平均收益",
    ]
    temp_df = temp_df.map(lambda x: x if "%" not in str(x) else x.replace("%", ""))
    temp_df["盈利概率"] = pd.to_numeric(temp_df["盈利概率"], errors="coerce")
    temp_df["平均收益"] = pd.to_numeric(temp_df["平均收益"], errors="coerce")
    return temp_df


def fund_individual_detail_info_xq(
    symbol: str = "000001", timeout: float = None
) -> pd.DataFrame:
    """
    雪球基金-交易规则
    https://danjuanfunds.com/djapi/fund/detail/675091
    :param symbol: 基金代码
    :type symbol: str
    :param timeout: choice of None or a positive float number
    :type timeout: float
    :return: 交易规则
    :rtype: pandas.DataFrame
    """
    url = f"https://danjuanfunds.com/djapi/fund/detail/{symbol}"
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/80.0.3987.149 Safari/537.36"
    }
    r = requests.get(url, headers=headers, timeout=timeout)
    json_data = r.json()["data"]
    combined_df = None
    rate_type_dict = {
        "declare_rate_table": "买入规则",
        "withdraw_rate_table": "卖出规则",
        "other_rate_table": "其他费用",
    }
    for k, v in rate_type_dict.items():
        temp_df = pd.DataFrame.from_dict(json_data["fund_rates"][k], orient="columns")
        temp_df["rate_type"] = v
        temp_df = temp_df[
            [
                "rate_type",
                "name",
                "value",
            ]
        ]
        temp_df.columns = [
            "费用类型",
            "条件或名称",
            "费用",
        ]
        combined_df = pd.concat(objs=[combined_df, temp_df], ignore_index=True)
        combined_df["费用"] = pd.to_numeric(combined_df["费用"], errors="coerce")
    return combined_df


def fund_individual_detail_hold_xq(
    symbol: str = "002804", date: str = "20231231", timeout: float = None
) -> pd.DataFrame:
    """
    雪球基金-持仓
    https://danjuanfunds.com/rn/fund-detail/archive?id=103&code=002804
    :param symbol: 基金代码
    :type symbol: str
    :param date: 财报日期
    :type date: str
    :param timeout: choice of None or a positive float number
    :type timeout: float
    :return: 雪球基金-持仓
    :rtype: pandas.DataFrame
    """
    url = "https://danjuanfunds.com/djapi/fundx/base/fund/record/asset/percent"
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/80.0.3987.149 Safari/537.36"
    }
    params = {
        "fund_code": f"{symbol}",
        "report_date": f"{'-'.join([date[:4], date[4:6], date[6:]])}",
    }
    r = requests.get(url, headers=headers, params=params, timeout=timeout)
    data_json = r.json()
    temp_df = pd.DataFrame.from_dict(data_json["data"]["chart_list"], orient="columns")
    temp_df = temp_df[
        [
            "type_desc",
            "percent",
        ]
    ]
    temp_df.columns = [
        "资产类型",
        "仓位占比",
    ]
    temp_df["仓位占比"] = pd.to_numeric(temp_df["仓位占比"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    fund_individual_basic_info_xq_df = fund_individual_basic_info_xq(symbol="000005")
    print(fund_individual_basic_info_xq_df)

    fund_individual_achievement_xq_df = fund_individual_achievement_xq(symbol="000001")
    print(fund_individual_achievement_xq_df)

    fund_individual_analysis_xq_df = fund_individual_analysis_xq(symbol="000001")
    print(fund_individual_analysis_xq_df)

    fund_individual_profit_probability_xq_df = fund_individual_profit_probability_xq(
        symbol="000001"
    )
    print(fund_individual_profit_probability_xq_df)

    fund_individual_detail_info_xq_df = fund_individual_detail_info_xq(symbol="000001")
    print(fund_individual_detail_info_xq_df)

    fund_individual_detail_hold_xq_df = fund_individual_detail_hold_xq(
        symbol="002804", date="20231231"
    )
    print(fund_individual_detail_hold_xq_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2023/11/11 16:30
Desc: 东方财富-基金
"""
from io import StringIO

import pandas as pd
import requests


def fund_aum_em() -> pd.DataFrame:
    """
    东方财富-基金-基金公司排名列表
    https://fund.eastmoney.com/Company/lsgm.html
    :return: 基金公司排名列表
    :rtype: pandas.DataFrame
    """
    url = 'https://fund.eastmoney.com/Company/home/gspmlist'
    params = {
        'fundType': '0'
    }
    r = requests.get(url, params=params)
    temp_df = pd.read_html(StringIO(r.text))[0]
    del temp_df['相关链接']
    del temp_df['天相评级']
    temp_df.columns = ['序号', '基金公司', '成立时间', '全部管理规模', '全部基金数', '全部经理数']
    expanded_df = temp_df['全部管理规模'].str.split(' ', expand=True)
    temp_df['全部管理规模'] = expanded_df.iloc[:, 0].str.replace(",", "")
    temp_df['更新日期'] = expanded_df.iloc[:, 1]
    temp_df['全部管理规模'] = pd.to_numeric(temp_df['全部管理规模'], errors="coerce")
    temp_df['全部基金数'] = pd.to_numeric(temp_df['全部基金数'], errors="coerce")
    temp_df['全部经理数'] = pd.to_numeric(temp_df['全部经理数'], errors="coerce")
    temp_df['成立时间'] = pd.to_datetime(temp_df['成立时间'], errors="coerce").dt.date
    return temp_df


def fund_aum_trend_em() -> pd.DataFrame:
    """
    东方财富-基金-基金市场管理规模走势图
    https://fund.eastmoney.com/Company/default.html
    :return: 基金市场管理规模走势图
    :rtype: pandas.DataFrame
    """
    url = 'https://fund.eastmoney.com/Company/home/GetFundTotalScaleForChart'
    payload = {
        'fundType': '0'
    }
    r = requests.get(url, data=payload)
    data_json = r.json()
    temp_df = pd.DataFrame()
    temp_df['date'] = data_json['x']
    temp_df['value'] = data_json['y']
    temp_df['date'] = pd.to_datetime(temp_df['date'], errors="coerce").dt.date
    temp_df['value'] = pd.to_numeric(temp_df['value'], errors="coerce")
    return temp_df


def fund_aum_hist_em(year: str = "2023") -> pd.DataFrame:
    """
    东方财富-基金-基金公司历年管理规模排行列表
    https://fund.eastmoney.com/Company/lsgm.html
    :return: 基金公司历年管理规模排行列表
    :rtype: pandas.DataFrame
    """
    url = 'https://fund.eastmoney.com/Company/home/HistoryScaleTable'
    params = {
        'year': year
    }
    r = requests.get(url, params=params)
    temp_df = pd.read_html(StringIO(r.text))[0]
    temp_df.columns = ['序号', '基金公司', '总规模', '股票型', '混合型', '债券型', '指数型', 'QDII', '货币型']
    temp_df['总规模'] = pd.to_numeric(temp_df['总规模'], errors="coerce")
    temp_df['股票型'] = pd.to_numeric(temp_df['股票型'], errors="coerce")
    temp_df['混合型'] = pd.to_numeric(temp_df['混合型'], errors="coerce")
    temp_df['债券型'] = pd.to_numeric(temp_df['债券型'], errors="coerce")
    temp_df['指数型'] = pd.to_numeric(temp_df['指数型'], errors="coerce")
    temp_df['QDII'] = pd.to_numeric(temp_df['QDII'], errors="coerce")
    temp_df['货币型'] = pd.to_numeric(temp_df['货币型'], errors="coerce")
    return temp_df


if __name__ == '__main__':
    fund_aum_em_df = fund_aum_em()
    print(fund_aum_em_df)

    fund_aum_trend_em_df = fund_aum_trend_em()
    print(fund_aum_trend_em_df)

    fund_em_aum_hist_df = fund_aum_hist_em(year="2023")
    print(fund_em_aum_hist_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/3/25 16:30
Desc: 新浪财经-交易日历
https://finance.sina.com.cn/realstock/company/klc_td_sh.txt
此处可以用来更新 calendar.json 文件，注意末尾没有 "," 号
"""

import datetime

import pandas as pd
import requests
import py_mini_racer

from byteapp_ak.stock.cons import hk_js_decode


def tool_trade_date_hist_sina() -> pd.DataFrame:
    """
    新浪财经-交易日历-历史数据
    https://finance.sina.com.cn/realstock/company/klc_td_sh.txt
    :return: 交易日历
    :rtype: pandas.DataFrame
    """
    url = "https://finance.sina.com.cn/realstock/company/klc_td_sh.txt"
    r = requests.get(url)
    js_code = py_mini_racer.MiniRacer()
    js_code.eval(hk_js_decode)
    dict_list = js_code.call("d", r.text.split("=")[1].split(";")[0].replace('"', ""))
    temp_df = pd.DataFrame(dict_list)
    temp_df.columns = ["trade_date"]
    temp_df["trade_date"] = pd.to_datetime(temp_df["trade_date"]).dt.date
    temp_list = temp_df["trade_date"].to_list()
    # 该日期是交易日，但是在新浪返回的交易日历缺失该日期，这里补充上
    temp_list.append(datetime.date(year=1992, month=5, day=4))
    temp_list.sort()
    temp_df = pd.DataFrame(temp_list, columns=["trade_date"])
    return temp_df


if __name__ == "__main__":
    tool_trade_date_hist_df = tool_trade_date_hist_sina()
    print(tool_trade_date_hist_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2023/3/9 20:26
Desc: 百度股市通-港股-财务报表-估值数据
https://gushitong.baidu.com/stock/hk-06969
"""
import http.client
import json
import urllib

import pandas as pd


def stock_hk_valuation_baidu(
    symbol: str = "06969", indicator: str = "总市值", period: str = "近一年"
) -> pd.DataFrame:
    """
    百度股市通-港股-财务报表-估值数据
    https://gushitong.baidu.com/stock/hk-06969
    :param symbol: 股票代码
    :type symbol: str
    :param indicator: choice of {"总市值", "市盈率(TTM)", "市盈率(静)", "市净率", "市现率"}
    :type indicator: str
    :param period: choice of {"近一年", "近三年", "近五年", "近十年", "全部"}
    :type period: str
    :return: 估值数据
    :rtype: pandas.DataFrame
    """
    params = {
        "srcid": "51171",
        "code": symbol,
        "market": "hk",
        "tag": indicator,
        "chart_select": period,
        "skip_industry": "0",
        "finClientType": "pc",
    }
    conn = http.client.HTTPSConnection("finance.pae.baidu.com")
    conn.request("GET", f"/selfselect/openapi?{urllib.parse.urlencode(params)}")
    r = conn.getresponse()
    data_json = json.loads(r.read())
    temp_df = pd.DataFrame(data_json["Result"]["chartInfo"][0]["body"])
    temp_df.columns = ["date", "value"]
    temp_df["date"] = pd.to_datetime(temp_df["date"]).dt.date
    temp_df["value"] = pd.to_numeric(temp_df["value"])
    return temp_df


if __name__ == "__main__":
    stock_hk_valuation_baidu_df = stock_hk_valuation_baidu(
        symbol="00700", indicator="总市值", period="近五年"
    )
    print(stock_hk_valuation_baidu_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2023/8/31 23:00
Desc: 金十数据-比特币持仓报告
https://datacenter.jin10.com/dc_report?name=bitcoint
"""
import pandas as pd
import requests


def crypto_bitcoin_hold_report():
    """
    金十数据-比特币持仓报告
    https://datacenter.jin10.com/dc_report?name=bitcoint
    :return: 比特币持仓报告
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-api.jin10.com/bitcoin_treasuries/list"
    headers = {
        "X-App-Id": "lnFP5lxse24wPgtY",
        "X-Version": "1.0.0",
    }
    r = requests.get(url, headers=headers)

    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"]["values"])
    temp_df.columns = [
        "代码",
        "公司名称-英文",
        "国家/地区",
        "市值",
        "比特币占市值比重",
        "持仓成本",
        "持仓占比",
        "持仓量",
        "当日持仓市值",
        "查询日期",
        "公告链接",
        "_",
        "分类",
        "倍数",
        "_",
        "公司名称-中文",
    ]
    temp_df = temp_df[
        [
            "代码",
            "公司名称-英文",
            "公司名称-中文",
            "国家/地区",
            "市值",
            "比特币占市值比重",
            "持仓成本",
            "持仓占比",
            "持仓量",
            "当日持仓市值",
            "查询日期",
            "公告链接",
            "分类",
            "倍数",
        ]
    ]
    temp_df["市值"] = pd.to_numeric(temp_df["市值"], errors="coerce")
    temp_df["比特币占市值比重"] = pd.to_numeric(temp_df["比特币占市值比重"], errors="coerce")
    temp_df["持仓成本"] = pd.to_numeric(temp_df["持仓成本"], errors="coerce")
    temp_df["持仓占比"] = pd.to_numeric(temp_df["持仓占比"], errors="coerce")
    temp_df["持仓量"] = pd.to_numeric(temp_df["持仓量"], errors="coerce")
    temp_df["当日持仓市值"] = pd.to_numeric(temp_df["当日持仓市值"], errors="coerce")
    temp_df["倍数"] = pd.to_numeric(temp_df["倍数"], errors="coerce")
    temp_df["查询日期"] = pd.to_datetime(temp_df["查询日期"], errors="coerce").dt.date
    return temp_df


if __name__ == "__main__":
    crypto_bitcoin_hold_report_df = crypto_bitcoin_hold_report()
    print(crypto_bitcoin_hold_report_df)



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2023/11/9 14:00
Desc: 中证商品指数
http://www.ccidx.com/
"""
from io import BytesIO

import pandas as pd
import requests


def futures_index_ccidx(symbol: str = "中证商品期货指数") -> pd.DataFrame:
    """
    中证商品指数-商品指数-日频率
    http://www.ccidx.com/index.html
    :param symbol: choice of {"中证商品期货指数", "中证商品期货价格指数"}
    :type symbol: str
    :return: 商品指数-日频率
    :rtype: pandas.DataFrame
    """
    futures_index_map = {
        "中证商品期货指数": "100001.CCI",
        "中证商品期货价格指数": "000001.CCI",
    }
    url = "http://www.ccidx.com/front/ajax_downZSHQ.do"
    params = {"indexCode": futures_index_map[symbol]}
    r = requests.get(url, params=params)
    temp_df = pd.read_excel(BytesIO(r.content), header=1, engine="openpyxl")
    temp_df.columns = [
        "日期",
        "指数代码",
        "指数中文全称",
        "指数中文简称",
        "指数英文全称",
        "指数英文简称",
        "开盘",
        "最高",
        "最低",
        "收盘",
        "结算",
        "涨跌",
        "涨跌幅",
    ]
    temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date
    temp_df["开盘"] = pd.to_numeric(temp_df["开盘"], errors="coerce")
    temp_df["最高"] = pd.to_numeric(temp_df["最高"], errors="coerce")
    temp_df["最低"] = pd.to_numeric(temp_df["最低"], errors="coerce")
    temp_df["收盘"] = pd.to_numeric(temp_df["收盘"], errors="coerce")
    temp_df["结算"] = pd.to_numeric(temp_df["结算"], errors="coerce")
    temp_df["涨跌"] = pd.to_numeric(temp_df["涨跌"], errors="coerce")
    temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"], errors="coerce")
    temp_df.sort_values(by=['日期'], inplace=True)
    temp_df.reset_index(inplace=True, drop=True)
    return temp_df


def futures_index_min_ccidx(symbol: str = "中证监控油脂油料期货指数") -> pd.DataFrame:
    """
    中证商品指数-商品指数-分时数据
    http://www.ccidx.com/index.html
    :param symbol: choice of {"中证商品期货指数", "中证商品期货价格指数", "中证监控油脂油料期货指数", "中证监控软商品期货指数",  "中证监控能化期货指数", "中证监控钢铁期货指数"}
    :type symbol: str
    :return: 商品指数-分时数据
    :rtype: pandas.DataFrame
    """
    futures_index_map = {
        "中证商品期货指数": ["100001.CCI", "0"],
        "中证商品期货价格指数": ["000001.CCI", "1"],
        "中证监控油脂油料期货指数": ["606005.CCI", "2"],
        "中证监控软商品期货指数": ["606008.CCI", "3"],
        "中证监控能化期货指数": ["606010.CCI", "4"],
        "中证监控钢铁期货指数": ["606011.CCI", "5"],
    }
    url = "http://www.ccidx.com/cscidx/csciAction/loadTimeData"
    params = {"r": "0.08644997232349438"}
    payload = {
        "indexCode": futures_index_map[symbol][0],
        "indexType": futures_index_map[symbol][1],
        "pointer": "all",
    }
    headers = {
        "Accept": "application/json, text/javascript, */*; q=0.01",
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Content-Length": "44",
        "Connection": "keep-alive",
        "Content-Type": "application/x-www-form-urlencoded; charset=UTF-8",
        "Host": "www.ccidx.com",
        "Origin": "http://www.ccidx.com",
        "Pragma": "no-cache",
        "Proxy-Connection": "keep-alive",
        "Referer": "http://www.ccidx.com/cscidx/quote1",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/92.0.4515.159 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    r = requests.post(url, params=params, data=payload, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(
        [data_json["dataMap"]["axisList"], data_json["dataMap"]["lineList"]]
    ).T
    temp_df.columns = [
        "datetime",
        "value",
    ]
    temp_df["value"] = pd.to_numeric(temp_df["value"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    futures_index_ccidx_df = futures_index_ccidx(symbol="中证商品期货指数")
    print(futures_index_ccidx_df)

    futures_index_min_ccidx_df = futures_index_min_ccidx(symbol="中证商品期货指数")
    print(futures_index_min_ccidx_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/7/24 13:00
Desc: 东方财富网-数据中心-开放基金排行
https://fund.eastmoney.com/data/fundranking.html
名词解释
https://help.1234567.com.cn/list_236.html
"""

from datetime import datetime, date

import pandas as pd
import requests

from byteapp_ak.utils import demjson


def __one_year_ago(date_str: str) -> date:
    # 将字符串格式的日期转换为date对象
    given_date = date(int(date_str[0:4]), int(date_str[4:6]), int(date_str[6:8]))

    try:
        # 尝试直接设置为前一年，保持相同的月和日
        one_year_before = given_date.replace(year=given_date.year - 1)
    except ValueError:
        # 如果前一年没有相同的月和日（比如2月29日），则设置为2月28日
        one_year_before = given_date.replace(year=given_date.year - 1, day=28)

    return one_year_before


def fund_open_fund_rank_em(symbol: str = "全部") -> pd.DataFrame:
    """
    东方财富网-数据中心-开放基金排行
    https://fund.eastmoney.com/data/fundranking.html
    :param symbol: choice of {"全部", "股票型", "混合型", "债券型", "指数型", "QDII", "FOF"}
    :type symbol: str
    :return: 开放基金排行
    :rtype: pandas.DataFrame
    """
    current_date = datetime.now().date().isoformat()
    last_date = __one_year_ago(current_date.replace("-", "")).isoformat()
    url = "https://fund.eastmoney.com/data/rankhandler.aspx"
    type_map = {
        "全部": ["all", "1nzf"],
        "股票型": ["gp", "1nzf"],
        "混合型": ["hh", "1nzf"],
        "债券型": ["zq", "1nzf"],
        "指数型": ["zs", "1nzf"],
        "QDII": ["qdii", "1nzf"],
        "LOF": ["lof", "1nzf"],
        "FOF": ["fof", "1nzf"],
    }
    params = {
        "op": "ph",
        "dt": "kf",
        "ft": type_map[symbol][0],
        "rs": "",
        "gs": "0",
        "sc": type_map[symbol][1],
        "st": "desc",
        "sd": last_date,
        "ed": current_date,
        "qdii": "",
        "tabSubtype": ",,,,,",
        "pi": "1",
        "pn": "20000",
        "dx": "1",
        "v": "0.1591891419018292",
    }
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/81.0.4044.138 Safari/537.36",
        "Referer": "https://fund.eastmoney.com/fundguzhi.html",
    }
    r = requests.get(url, params=params, headers=headers)
    data_text = r.text
    data_json = demjson.decode(data_text[data_text.find("{") : -1])
    temp_df = pd.DataFrame(data_json["datas"])
    temp_df = temp_df.iloc[:, 0].str.split(",", expand=True)
    temp_df.reset_index(inplace=True)
    temp_df["index"] = list(range(1, len(temp_df) + 1))
    temp_df.columns = [
        "序号",
        "基金代码",
        "基金简称",
        "_",
        "日期",
        "单位净值",
        "累计净值",
        "日增长率",
        "近1周",
        "近1月",
        "近3月",
        "近6月",
        "近1年",
        "近2年",
        "近3年",
        "今年来",
        "成立来",
        "_",
        "_",
        "自定义",
        "_",
        "手续费",
        "_",
        "_",
        "_",
        "_",
    ]
    temp_df = temp_df[
        [
            "序号",
            "基金代码",
            "基金简称",
            "日期",
            "单位净值",
            "累计净值",
            "日增长率",
            "近1周",
            "近1月",
            "近3月",
            "近6月",
            "近1年",
            "近2年",
            "近3年",
            "今年来",
            "成立来",
            "自定义",
            "手续费",
        ]
    ]
    temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date
    temp_df["单位净值"] = pd.to_numeric(temp_df["单位净值"], errors="coerce")
    temp_df["累计净值"] = pd.to_numeric(temp_df["累计净值"], errors="coerce")
    temp_df["日增长率"] = pd.to_numeric(temp_df["日增长率"], errors="coerce")
    temp_df["近1周"] = pd.to_numeric(temp_df["近1周"], errors="coerce")
    temp_df["近1月"] = pd.to_numeric(temp_df["近1月"], errors="coerce")
    temp_df["近3月"] = pd.to_numeric(temp_df["近3月"], errors="coerce")
    temp_df["近6月"] = pd.to_numeric(temp_df["近6月"], errors="coerce")
    temp_df["近1年"] = pd.to_numeric(temp_df["近1年"], errors="coerce")
    temp_df["近2年"] = pd.to_numeric(temp_df["近2年"], errors="coerce")
    temp_df["近3年"] = pd.to_numeric(temp_df["近3年"], errors="coerce")
    temp_df["今年来"] = pd.to_numeric(temp_df["今年来"], errors="coerce")
    temp_df["成立来"] = pd.to_numeric(temp_df["成立来"], errors="coerce")
    temp_df["自定义"] = pd.to_numeric(temp_df["自定义"], errors="coerce")
    return temp_df


def fund_exchange_rank_em() -> pd.DataFrame:
    """
    东方财富网-数据中心-场内交易基金排行
    https://fund.eastmoney.com/data/fbsfundranking.html
    :return: 场内交易基金数据
    :rtype: pandas.DataFrame
    """
    url = "https://fund.eastmoney.com/data/rankhandler.aspx"
    params = {
        "op": "ph",
        "dt": "fb",
        "ft": "ct",
        "rs": "",
        "gs": "0",
        "sc": "1nzf",
        "st": "desc",
        "pi": "1",
        "pn": "10000",
        "v": "0.1591891419018292",
    }
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/81.0.4044.138 Safari/537.36",
        "Referer": "https://fund.eastmoney.com/fundguzhi.html",
    }
    r = requests.get(url, params=params, headers=headers)
    text_data = r.text
    json_data = demjson.decode(text_data[text_data.find("{") : -1])
    temp_df = pd.DataFrame(json_data["datas"])
    temp_df = temp_df.iloc[:, 0].str.split(",", expand=True)
    temp_df.reset_index(inplace=True)
    temp_df["index"] = list(range(1, len(temp_df) + 1))
    temp_df.columns = [
        "序号",
        "基金代码",
        "基金简称",
        "_",
        "日期",
        "单位净值",
        "累计净值",
        "近1周",
        "近1月",
        "近3月",
        "近6月",
        "近1年",
        "近2年",
        "近3年",
        "今年来",
        "成立来",
        "成立日期",
        "_",
        "_",
        "_",
        "_",
        "_",
        "类型",
        "_",
    ]
    temp_df = temp_df[
        [
            "序号",
            "基金代码",
            "基金简称",
            "类型",
            "日期",
            "单位净值",
            "累计净值",
            "近1周",
            "近1月",
            "近3月",
            "近6月",
            "近1年",
            "近2年",
            "近3年",
            "今年来",
            "成立来",
            "成立日期",
        ]
    ]
    temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date
    temp_df["成立日期"] = pd.to_datetime(temp_df["成立日期"]).dt.date
    temp_df["单位净值"] = pd.to_numeric(temp_df["单位净值"], errors="coerce")
    temp_df["累计净值"] = pd.to_numeric(temp_df["累计净值"], errors="coerce")
    temp_df["近1周"] = pd.to_numeric(temp_df["近1周"], errors="coerce")
    temp_df["近1月"] = pd.to_numeric(temp_df["近1月"], errors="coerce")
    temp_df["近3月"] = pd.to_numeric(temp_df["近3月"], errors="coerce")
    temp_df["近6月"] = pd.to_numeric(temp_df["近6月"], errors="coerce")
    temp_df["近1年"] = pd.to_numeric(temp_df["近1年"], errors="coerce")
    temp_df["近2年"] = pd.to_numeric(temp_df["近2年"], errors="coerce")
    temp_df["近3年"] = pd.to_numeric(temp_df["近3年"], errors="coerce")
    temp_df["今年来"] = pd.to_numeric(temp_df["今年来"], errors="coerce")
    temp_df["成立来"] = pd.to_numeric(temp_df["成立来"], errors="coerce")
    return temp_df


def fund_money_rank_em() -> pd.DataFrame:
    """
    东方财富网-数据中心-货币型基金排行
    https://fund.eastmoney.com/data/hbxfundranking.html
    :return: 货币型基金排行
    :rtype: pandas.DataFrame
    """
    url = "https://api.fund.eastmoney.com/FundRank/GetHbRankList"
    params = {
        "intCompany": "0",
        "MinsgType": "",
        "IsSale": "1",
        "strSortCol": "SYL_1N",
        "orderType": "desc",
        "pageIndex": "1",
        "pageSize": "10000",
        "_": "1603867224251",
    }
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/81.0.4044.138 Safari/537.36",
        "Referer": "https://fund.eastmoney.com/fundguzhi.html",
    }
    r = requests.get(url, params=params, headers=headers)
    json_data = r.json()
    temp_df = pd.DataFrame(json_data["Data"])
    temp_df.reset_index(inplace=True)
    temp_df["index"] = list(range(1, len(temp_df) + 1))
    temp_df.columns = [
        "序号",
        "近1年",
        "近2年",
        "近3年",
        "近5年",
        "_",
        "_",
        "基金代码",
        "基金简称",
        "日期",
        "万份收益",
        "年化收益率7日",
        "_",
        "年化收益率14日",
        "年化收益率28日",
        "近1月",
        "近3月",
        "近6月",
        "今年来",
        "成立来",
        "_",
        "手续费",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
    ]
    temp_df = temp_df[
        [
            "序号",
            "基金代码",
            "基金简称",
            "日期",
            "万份收益",
            "年化收益率7日",
            "年化收益率14日",
            "年化收益率28日",
            "近1月",
            "近3月",
            "近6月",
            "近1年",
            "近2年",
            "近3年",
            "近5年",
            "今年来",
            "成立来",
            "手续费",
        ]
    ]
    temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date
    temp_df["万份收益"] = pd.to_numeric(temp_df["万份收益"], errors="coerce")
    temp_df["年化收益率7日"] = pd.to_numeric(temp_df["年化收益率7日"], errors="coerce")
    temp_df["年化收益率14日"] = pd.to_numeric(
        temp_df["年化收益率14日"], errors="coerce"
    )
    temp_df["年化收益率28日"] = pd.to_numeric(
        temp_df["年化收益率28日"], errors="coerce"
    )
    temp_df["近1月"] = pd.to_numeric(temp_df["近1月"], errors="coerce")
    temp_df["近3月"] = pd.to_numeric(temp_df["近3月"], errors="coerce")
    temp_df["近6月"] = pd.to_numeric(temp_df["近6月"], errors="coerce")
    temp_df["近1年"] = pd.to_numeric(temp_df["近1年"], errors="coerce")
    temp_df["近2年"] = pd.to_numeric(temp_df["近2年"], errors="coerce")
    temp_df["近3年"] = pd.to_numeric(temp_df["近3年"], errors="coerce")
    temp_df["近5年"] = pd.to_numeric(temp_df["近5年"], errors="coerce")
    temp_df["今年来"] = pd.to_numeric(temp_df["今年来"], errors="coerce")
    temp_df["成立来"] = pd.to_numeric(temp_df["成立来"], errors="coerce")
    return temp_df


def fund_lcx_rank_em() -> pd.DataFrame:
    """
    东方财富网-数据中心-理财基金排行
    # 该接口暂时没有数据
    https://fund.eastmoney.com/data/lcxfundranking.html#t;c0;r;sSYL_Z;ddesc;pn50;f;os1;
    :return: 理财基金排行
    :rtype: pandas.DataFrame
    """
    url = "https://api.fund.eastmoney.com/FundRank/GetLcRankList"
    params = {
        "intCompany": "0",
        "MinsgType": "undefined",
        "IsSale": "1",
        "strSortCol": "SYL_Z",
        "orderType": "desc",
        "pageIndex": "1",
        "pageSize": "50",
        "FBQ": "",
        "callback": "jQuery18303264654966943197_1603867158043",
        "_": "1603867224251",
    }
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/81.0.4044.138 Safari/537.36",
        "Referer": "https://fund.eastmoney.com/fundguzhi.html",
    }
    r = requests.get(url, params=params, headers=headers)
    try:
        data_json = r.json()
    except:  # noqa: E722
        return pd.DataFrame()
    temp_df = pd.DataFrame(data_json["Data"])
    temp_df.reset_index(inplace=True)
    temp_df["index"] = list(range(1, len(temp_df) + 1))
    temp_df.columns = [
        "序号",
        "近1周",
        "基金代码",
        "基金简称",
        "日期",
        "万份收益",
        "年化收益率-7日",
        "_",
        "年化收益率-14日",
        "年化收益率-28日",
        "近1月",
        "近3月",
        "近6月",
        "今年来",
        "成立来",
        "可购买",
        "手续费",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
    ]
    temp_df = temp_df[
        [
            "序号",
            "基金代码",
            "基金简称",
            "日期",
            "万份收益",
            "年化收益率-7日",
            "年化收益率-14日",
            "年化收益率-28日",
            "近1周",
            "近1月",
            "近3月",
            "近6月",
            "今年来",
            "成立来",
            "可购买",
            "手续费",
        ]
    ]
    return temp_df


def fund_hk_rank_em() -> pd.DataFrame:
    """
    东方财富网-数据中心-香港基金排行
    https://overseas.1234567.com.cn/FundList
    :return: 香港基金排行
    :rtype: pandas.DataFrame
    """
    format_date = datetime.now().date().isoformat()
    url = "https://overseas.1234567.com.cn/overseasapi/OpenApiHander.ashx"
    params = {
        "api": "HKFDApi",
        "m": "MethodFundList",
        "action": "1",
        "pageindex": "0",
        "pagesize": "5000",
        "dy": "1",
        "date1": format_date,
        "date2": format_date,
        "sortfield": "Y",
        "sorttype": "-1",
        "isbuy": "0",
        "_": "1610790553848",
    }
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/81.0.4044.138 Safari/537.36",
        "Referer": "https://fund.eastmoney.com/fundguzhi.html",
    }
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["Data"])
    temp_df.reset_index(inplace=True)
    temp_df["index"] = list(range(1, len(temp_df) + 1))
    temp_df.columns = [
        "序号",
        "_",
        "香港基金代码",
        "基金代码",
        "_",
        "基金简称",
        "可购买",
        "日期",
        "单位净值",
        "日增长率",
        "_",
        "近1周",
        "近1月",
        "近3月",
        "近6月",
        "近1年",
        "近2年",
        "近3年",
        "今年来",
        "成立来",
        "币种",
    ]
    temp_df = temp_df[
        [
            "序号",
            "基金代码",
            "基金简称",
            "币种",
            "日期",
            "单位净值",
            "日增长率",
            "近1周",
            "近1月",
            "近3月",
            "近6月",
            "近1年",
            "近2年",
            "近3年",
            "今年来",
            "成立来",
            "可购买",
            "香港基金代码",
        ]
    ]
    temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date
    temp_df["单位净值"] = pd.to_numeric(temp_df["单位净值"], errors="coerce")
    temp_df["日增长率"] = pd.to_numeric(temp_df["日增长率"], errors="coerce")
    temp_df["近1周"] = pd.to_numeric(temp_df["近1周"], errors="coerce")
    temp_df["近1月"] = pd.to_numeric(temp_df["近1月"], errors="coerce")
    temp_df["近3月"] = pd.to_numeric(temp_df["近3月"], errors="coerce")
    temp_df["近6月"] = pd.to_numeric(temp_df["近6月"], errors="coerce")
    temp_df["近1年"] = pd.to_numeric(temp_df["近1年"], errors="coerce")
    temp_df["近2年"] = pd.to_numeric(temp_df["近2年"], errors="coerce")
    temp_df["近3年"] = pd.to_numeric(temp_df["近3年"], errors="coerce")
    temp_df["今年来"] = pd.to_numeric(temp_df["今年来"], errors="coerce")
    temp_df["成立来"] = pd.to_numeric(temp_df["成立来"], errors="coerce")
    temp_df["成立来"] = pd.to_numeric(temp_df["成立来"], errors="coerce")
    temp_df["可购买"] = temp_df["可购买"].map(
        lambda x: "可购买" if x == "1" else "不可购买"
    )
    return temp_df


if __name__ == "__main__":
    fund_open_fund_rank_em_df = fund_open_fund_rank_em(symbol="全部")
    print(fund_open_fund_rank_em_df)

    fund_open_fund_rank_em_df = fund_open_fund_rank_em(symbol="股票型")
    print(fund_open_fund_rank_em_df)

    fund_open_fund_rank_em_df = fund_open_fund_rank_em(symbol="混合型")
    print(fund_open_fund_rank_em_df)

    fund_open_fund_rank_em_df = fund_open_fund_rank_em(symbol="债券型")
    print(fund_open_fund_rank_em_df)

    fund_open_fund_rank_em_df = fund_open_fund_rank_em(symbol="指数型")
    print(fund_open_fund_rank_em_df)

    fund_open_fund_rank_em_df = fund_open_fund_rank_em(symbol="QDII")
    print(fund_open_fund_rank_em_df)

    fund_open_fund_rank_em_df = fund_open_fund_rank_em(symbol="FOF")
    print(fund_open_fund_rank_em_df)

    fund_exchange_rank_em_df = fund_exchange_rank_em()
    print(fund_exchange_rank_em_df)

    fund_money_rank_em_df = fund_money_rank_em()
    print(fund_money_rank_em_df)

    fund_lcx_rank_em_df = fund_lcx_rank_em()
    print(fund_lcx_rank_em_df)

    fund_hk_rank_em_df = fund_hk_rank_em()
    print(fund_hk_rank_em_df)



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2023/7/6 17:00
Desc: 中证指数-所有指数-历史行情数据
https://www.csindex.com.cn/zh-CN/indices/index-detail/H30374#/indices/family/list?index_series=1
"""

import hashlib
import time
from functools import lru_cache

import pandas as pd
import requests


def __get_current_timestamp_ms() -> int:
    """
    生成时间戳
    :return: 时间戳
    :rtype: int
    """
    timestamp_seconds = time.time()
    timestamp_ms = int(timestamp_seconds * 1000)
    return timestamp_ms


def __md5_hash(input_string) -> str:
    """
    生成 md5 加密后的值
    :return: 生成 md5 加密后的值
    :rtype: str
    """
    md5 = hashlib.md5()
    md5.update(input_string.encode("utf-8"))
    return md5.hexdigest()


def __create_encode(
    act_time="1688635494326",
    authtoken="",
    gu_code="399808.SZ",
    pe_category="pb",
    type="pc",
    ver="new",
    version="2.2.7",
    year=-1,
) -> dict:
    """
    生成 post 密文，需要 JS 观察如下文件：
    https://funddb.cn/static/js/app.1c429c670c72542fb4fd.js
    :return: 生成 post 密文
    :rtype: str
    """
    input_string = f"{act_time}{authtoken}{gu_code}{pe_category}{type}{ver}{version}{year}EWf45rlv#kfsr@k#gfksgkr"
    hash_value = __md5_hash(input_string)
    l = hash_value  # noqa: E741
    c = l[29:31]
    d = l[2:4]
    f = l[5:6]
    h = l[26:27]
    m = l[6:8]
    v = l[1:2]
    y = l[0:2]
    k = l[6:8]
    w = l[8:9]
    x = l[30:31]
    P = l[11:14]
    z = l[11:12]
    j = l[2:5]
    q = l[9:11]
    H = l[23:25]
    O = l[31:32]  # noqa: E741
    C = l[25:27]
    E = l[9:11]
    A = l[27:29]
    T = l[17:19]
    F = l[26:27]
    U = l[12:14]
    S = l[25:26]
    R = l[16:19]
    K = l[17:21]
    I = l[18:19]  # noqa: E741
    D = l[21:23]
    _ = l[
        14:16
    ]  # $ is not a valid variable name in Python, so I replaced it with an underscore
    B = l[29:32]
    N = l[21:23]
    V = l[24:26]
    Y = l[16:17]

    def b(
        t,
        e,
        n,
        i,
        a,
        r,
        o,
        l,  # noqa: E741
        u,
        c,
        s,
        d,
        _,
        f,
        h,
        p,
        m,
        g,
        v,
        y,
        b,
        k,
        w,
        x,
        P,
        z,
        j,
        q,
        H,
        O,  # noqa: E741
        C,
        E,
        A,
    ):
        t["data"]["tirgkjfs"] = f
        t["data"]["abiokytke"] = _
        t["data"]["u54rg5d"] = e
        t["data"]["kf54ge7"] = q
        t["data"]["tiklsktr4"] = d
        t["data"]["lksytkjh"] = z
        t["data"]["sbnoywr"] = j
        t["data"]["bgd7h8tyu54"] = w
        t["data"]["y654b5fs3tr"] = C
        t["data"]["bioduytlw"] = n
        t["data"]["bd4uy742"] = P
        t["data"]["h67456y"] = o
        t["data"]["bvytikwqjk"] = s
        t["data"]["ngd4uy551"] = b
        t["data"]["bgiuytkw"] = v
        t["data"]["nd354uy4752"] = g
        t["data"]["ghtoiutkmlg"] = x
        t["data"]["bd24y6421f"] = i
        t["data"]["tbvdiuytk"] = l
        t["data"]["ibvytiqjek"] = p
        t["data"]["jnhf8u5231"] = A
        t["data"]["fjlkatj"] = E
        t["data"]["hy5641d321t"] = H
        t["data"]["iogojti"] = r
        t["data"]["ngd4yut78"] = a
        t["data"]["nkjhrew"] = c
        t["data"]["yt447e13f"] = O
        t["data"]["n3bf4uj7y7"] = k
        t["data"]["nbf4uj7y432"] = h
        t["data"]["yi854tew"] = u
        t["data"]["h13ey474"] = m
        t["data"]["quikgdky"] = y

    t = {"data": {}}

    b(
        t,
        d,
        f,
        V,
        U,
        S,
        R,
        Y,
        c,
        h,
        m,
        v,
        N,
        y,
        D,
        _,
        B,
        x,
        E,
        A,
        T,
        I,
        k,
        P,
        F,
        K,
        H,
        O,
        C,
        w,
        z,
        j,
        q,
    )
    return t["data"]


def stock_zh_index_hist_csindex(
    symbol: str = "000928",
    start_date: str = "20180526",
    end_date: str = "20240604",
) -> pd.DataFrame:
    """
    中证指数-具体指数-历史行情数据
    P.S. 只有收盘价，正常情况下不应使用该接口，除非指数只有中证网站有
    https://www.csindex.com.cn/zh-CN/indices/index-detail/H30374#/indices/family/detail?indexCode=H30374
    :param symbol: 指数代码; e.g., H30374
    :type symbol: str
    :param start_date: 开始日期
    :type start_date: str
    :param end_date: 结束日期
    :type end_date: str
    :return: 包含日期和收盘价的指数数据
    :rtype: pandas.DataFrame
    """
    url = "https://www.csindex.com.cn/csindex-home/perf/index-perf"
    params = {
        "indexCode": symbol,
        "startDate": start_date,
        "endDate": end_date,
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"])
    temp_df.columns = [
        "日期",
        "指数代码",
        "指数中文全称",
        "指数中文简称",
        "指数英文全称",
        "指数英文简称",
        "开盘",
        "最高",
        "最低",
        "收盘",
        "涨跌",
        "涨跌幅",
        "成交量",
        "成交金额",
        "样本数量",
        "滚动市盈率",
    ]
    temp_df["日期"] = pd.to_datetime(temp_df["日期"]).dt.date
    temp_df["开盘"] = pd.to_numeric(temp_df["开盘"], errors="coerce")
    temp_df["最高"] = pd.to_numeric(temp_df["最高"], errors="coerce")
    temp_df["最低"] = pd.to_numeric(temp_df["最低"], errors="coerce")
    temp_df["收盘"] = pd.to_numeric(temp_df["收盘"], errors="coerce")
    temp_df["涨跌"] = pd.to_numeric(temp_df["涨跌"], errors="coerce")
    temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"], errors="coerce")
    temp_df["成交量"] = pd.to_numeric(temp_df["成交量"], errors="coerce")
    temp_df["成交金额"] = pd.to_numeric(temp_df["成交金额"], errors="coerce")
    temp_df["样本数量"] = pd.to_numeric(temp_df["样本数量"], errors="coerce")
    temp_df["滚动市盈率"] = pd.to_numeric(temp_df["滚动市盈率"], errors="coerce")
    return temp_df


def stock_zh_index_value_csindex(symbol: str = "H30374") -> pd.DataFrame:
    """
    中证指数-指数估值数据
    https://www.csindex.com.cn/zh-CN/indices/index-detail/H30374#/indices/family/detail?indexCode=H30374
    :param symbol: 指数代码; e.g., H30374
    :type symbol: str
    :return: 指数估值数据
    :rtype: pandas.DataFrame
    """
    url = (
        f"https://csi-web-dev.oss-cn-shanghai-finance-1-pub.aliyuncs.com/static/"
        f"html/csindex/public/uploads/file/autofile/indicator/{symbol}indicator.xls"
    )
    temp_df = pd.read_excel(url)
    temp_df.columns = [
        "日期",
        "指数代码",
        "指数中文全称",
        "指数中文简称",
        "指数英文全称",
        "指数英文简称",
        "市盈率1",
        "市盈率2",
        "股息率1",
        "股息率2",
    ]
    temp_df["日期"] = pd.to_datetime(
        temp_df["日期"], format="%Y%m%d", errors="coerce"
    ).dt.date
    temp_df["市盈率1"] = pd.to_numeric(temp_df["市盈率1"], errors="coerce")
    temp_df["市盈率2"] = pd.to_numeric(temp_df["市盈率2"], errors="coerce")
    temp_df["股息率1"] = pd.to_numeric(temp_df["股息率1"], errors="coerce")
    temp_df["股息率2"] = pd.to_numeric(temp_df["股息率2"], errors="coerce")
    return temp_df


@lru_cache()
def index_value_name_funddb() -> pd.DataFrame:
    """
    funddb-指数估值-指数代码
    https://funddb.cn/site/index
    :return: pandas.DataFrame
    :rtype: 指数代码
    """
    url = "https://api.jiucaishuo.com/v2/guzhi/showcategory"
    get_current_timestamp_ms_str = __get_current_timestamp_ms()
    encode_params = __create_encode(
        act_time=str(get_current_timestamp_ms_str),
        authtoken="",
        gu_code="",
        pe_category="",
        type="pc",
        ver="",
        version="2.2.7",
        year="",
    )
    payload = {
        "type": "pc",
        "version": "2.2.7",
        "authtoken": "",
        "act_time": str(get_current_timestamp_ms_str),
    }
    payload.update(encode_params)
    r = requests.post(url, json=payload)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"]["right_list"])
    temp_df.columns = [
        "指数开始时间",
        "-",
        "指数名称",
        "指数代码",
        "最新PE",
        "最新PB",
        "PE分位",
        "PB分位",
        "股息率",
        "-",
        "-",
        "-",
        "更新时间",
        "股息率分位",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
        "-",
    ]
    temp_df = temp_df[
        [
            "指数名称",
            "最新PE",
            "PE分位",
            "最新PB",
            "PB分位",
            "股息率",
            "股息率分位",
            "指数代码",
            "指数开始时间",
            "更新时间",
        ]
    ]
    temp_df["指数开始时间"] = pd.to_datetime(
        temp_df["指数开始时间"], errors="coerce"
    ).dt.date
    temp_df["最新PE"] = pd.to_numeric(temp_df["最新PE"], errors="coerce")
    temp_df["PE分位"] = pd.to_numeric(temp_df["PE分位"], errors="coerce")
    temp_df["最新PB"] = pd.to_numeric(temp_df["最新PB"], errors="coerce")
    temp_df["PB分位"] = pd.to_numeric(temp_df["PB分位"], errors="coerce")
    temp_df["股息率"] = pd.to_numeric(temp_df["股息率"], errors="coerce")
    temp_df["股息率分位"] = pd.to_numeric(temp_df["股息率分位"], errors="coerce")
    return temp_df


def index_value_hist_funddb(
    symbol: str = "大盘成长", indicator: str = "市盈率"
) -> pd.DataFrame:
    """
    funddb-指数估值-估值信息
    https://funddb.cn/site/index
    :param symbol: 指数名称; 通过调用 ak.index_value_name_funddb() 来获取
    :type symbol: str
    :param indicator: choice of {'市盈率', '市净率', '股息率', '风险溢价'}
    :type indicator: str
    :return: 估值信息
    :rtype: pandas.DataFrame
    """
    indicator_map = {
        "市盈率": "pe",
        "市净率": "pb",
        "股息率": "xilv",
        "风险溢价": "fed",
    }
    index_value_name_funddb_df = index_value_name_funddb()
    name_code_map = dict(
        zip(
            index_value_name_funddb_df["指数名称"],
            index_value_name_funddb_df["指数代码"],
        )
    )
    url = "https://api.jiucaishuo.com/v2/guzhi/newtubiaolinedata"
    get_current_timestamp_ms_str = __get_current_timestamp_ms()
    encode_params = __create_encode(
        act_time=str(get_current_timestamp_ms_str),
        authtoken="",
        gu_code=name_code_map[symbol],
        pe_category=indicator_map[indicator],
        type="pc",
        ver="new",
        version="2.2.7",
        year=-1,
    )
    payload = {
        "gu_code": name_code_map[symbol],
        "pe_category": indicator_map[indicator],
        "year": -1,
        "ver": "new",
        "type": "pc",
        "version": "2.2.7",
        "authtoken": "",
        "act_time": str(get_current_timestamp_ms_str),
    }
    payload.update(encode_params)
    r = requests.post(url, json=payload)
    data_json = r.json()
    big_df = pd.DataFrame()
    temp_df = pd.DataFrame(
        data_json["data"]["tubiao"]["series"][0]["data"],
        columns=["timestamp", "value"],
    )
    big_df["日期"] = (
        pd.to_datetime(temp_df["timestamp"], unit="ms", utc=True)
        .dt.tz_convert("Asia/Shanghai")
        .dt.date
    )
    big_df["平均值"] = pd.to_numeric(temp_df["value"])
    big_df[indicator] = pd.to_numeric(
        [item[1] for item in data_json["data"]["tubiao"]["series"][1]["data"]]
    )
    big_df["最低30"] = pd.to_numeric(
        [item[1] for item in data_json["data"]["tubiao"]["series"][2]["data"]]
    )
    big_df["最低10"] = pd.to_numeric(
        [item[1] for item in data_json["data"]["tubiao"]["series"][3]["data"]]
    )
    big_df["最高30"] = pd.to_numeric(
        [item[1] for item in data_json["data"]["tubiao"]["series"][4]["data"]]
    )
    big_df["最高10"] = pd.to_numeric(
        [item[1] for item in data_json["data"]["tubiao"]["series"][5]["data"]]
    )
    return big_df


if __name__ == "__main__":
    stock_zh_index_hist_csindex_df = stock_zh_index_hist_csindex(
        symbol="000928", start_date="20100101", end_date="20240604"
    )
    print(stock_zh_index_hist_csindex_df)

    stock_zh_index_value_csindex_df = stock_zh_index_value_csindex(symbol="H30374")
    print(stock_zh_index_value_csindex_df)

    index_value_name_funddb_df = index_value_name_funddb()
    print(index_value_name_funddb_df)

    index_value_hist_funddb_df = index_value_hist_funddb(
        symbol="大盘成长", indicator="市盈率"
    )
    print(index_value_hist_funddb_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/10/16 21:30
Desc: 新浪财经-期货-成交持仓
https://vip.stock.finance.sina.com.cn/q/view/vFutures_Positions_cjcc.php
"""

from io import StringIO

import pandas as pd
import requests


def futures_hold_pos_sina(
    symbol: str = "成交量", contract: str = "IC2403", date: str = "20240223"
) -> pd.DataFrame:
    """
    新浪财经-期货-成交持仓
    https://vip.stock.finance.sina.com.cn/q/view/vFutures_Positions_cjcc.php
    :param symbol: choice of {"成交量", "多单持仓", "空单持仓"}
    :type symbol: str
    :param contract: 期货合约
    :type contract: str
    :param date: 查询日期
    :type date: str
    :return: 成交持仓
    :rtype: pandas.DataFrame
    """
    date = "-".join([date[:4], date[4:6], date[6:]])
    url = "https://vip.stock.finance.sina.com.cn/q/view/vFutures_Positions_cjcc.php"
    params = {"t_breed": contract, "t_date": date}
    r = requests.get(url, params=params)
    if symbol == "成交量":
        temp_df = pd.read_html(StringIO(r.text))[2].iloc[:-1, :]
        temp_df["名次"] = pd.to_numeric(temp_df["名次"], errors="coerce")
        temp_df["成交量"] = pd.to_numeric(temp_df["成交量"], errors="coerce")
        temp_df["比上交易增减"] = pd.to_numeric(
            temp_df["比上交易增减"], errors="coerce"
        )
        return temp_df
    elif symbol == "多单持仓":
        temp_df = pd.read_html(StringIO(r.text))[3].iloc[:-1, :]
        temp_df["名次"] = pd.to_numeric(temp_df["名次"], errors="coerce")
        temp_df["多单持仓"] = pd.to_numeric(temp_df["多单持仓"], errors="coerce")
        temp_df["比上交易增减"] = pd.to_numeric(
            temp_df["比上交易增减"], errors="coerce"
        )
        return temp_df
    elif symbol == "空单持仓":
        temp_df = pd.read_html(StringIO(r.text))[4].iloc[:-1, :]
        temp_df["名次"] = pd.to_numeric(temp_df["名次"], errors="coerce")
        temp_df["空单持仓"] = pd.to_numeric(temp_df["空单持仓"], errors="coerce")
        temp_df["比上交易增减"] = pd.to_numeric(
            temp_df["比上交易增减"], errors="coerce"
        )
        return temp_df


if __name__ == "__main__":
    futures_hold_pos_sina_df = futures_hold_pos_sina(
        symbol="成交量", contract="OI2501", date="20241016"
    )
    print(futures_hold_pos_sina_df)

    futures_hold_pos_sina_df = futures_hold_pos_sina(
        symbol="多单持仓", contract="OI2501", date="20241016"
    )
    print(futures_hold_pos_sina_df)

    futures_hold_pos_sina_df = futures_hold_pos_sina(
        symbol="空单持仓", contract="OI2501", date="20241016"
    )
    print(futures_hold_pos_sina_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2022/11/12 21:27
Desc: 东方财富-数据中心-年报季报
东方财富-数据中心-年报季报-业绩快报-业绩报表
https://data.eastmoney.com/bbsj/202003/yjbb.html
"""
import pandas as pd
import requests
from tqdm import tqdm


def stock_yjbb_em(date: str = "20200331") -> pd.DataFrame:
    """
    东方财富-数据中心-年报季报-业绩快报-业绩报表
    https://data.eastmoney.com/bbsj/202003/yjbb.html
    :param date: "20200331", "20200630", "20200930", "20201231"; 从 20100331 开始
    :type date: str
    :return: 业绩报表
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        'sortColumns': 'UPDATE_DATE,SECURITY_CODE',
        'sortTypes': '-1,-1',
        'pageSize': '500',
        'pageNumber': '1',
        'reportName': 'RPT_LICO_FN_CPD',
        'columns': 'ALL',
        'filter': f"(REPORTDATE='{'-'.join([date[:4], date[4:6], date[6:]])}')",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    page_num = data_json["result"]["pages"]
    big_df = pd.DataFrame()
    for page in tqdm(range(1, page_num + 1), leave=False):
        params.update({
            'pageNumber': page,
        })
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat([big_df, temp_df], ignore_index=True)

    big_df.reset_index(inplace=True)
    big_df["index"] = range(1, len(big_df) + 1)
    big_df.columns = [
        "序号",
        "股票代码",
        "股票简称",
        "_",
        "_",
        "_",
        "_",
        "最新公告日期",
        "_",
        "每股收益",
        "_",
        "营业收入-营业收入",
        "净利润-净利润",
        "净资产收益率",
        "营业收入-同比增长",
        "净利润-同比增长",
        "每股净资产",
        "每股经营现金流量",
        "销售毛利率",
        "营业收入-季度环比增长",
        "净利润-季度环比增长",
        "_",
        "_",
        "所处行业",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
        "_",
    ]
    big_df = big_df[
        [
            "序号",
            "股票代码",
            "股票简称",
            "每股收益",
            "营业收入-营业收入",
            "营业收入-同比增长",
            "营业收入-季度环比增长",
            "净利润-净利润",
            "净利润-同比增长",
            "净利润-季度环比增长",
            "每股净资产",
            "净资产收益率",
            "每股经营现金流量",
            "销售毛利率",
            "所处行业",
            "最新公告日期",
        ]
    ]
    big_df['每股收益'] = pd.to_numeric(big_df['每股收益'], errors="coerce")
    big_df['营业收入-营业收入'] = pd.to_numeric(big_df['营业收入-营业收入'], errors="coerce")
    big_df['营业收入-同比增长'] = pd.to_numeric(big_df['营业收入-同比增长'], errors="coerce")
    big_df['营业收入-季度环比增长'] = pd.to_numeric(big_df['营业收入-季度环比增长'], errors="coerce")
    big_df['净利润-净利润'] = pd.to_numeric(big_df['净利润-净利润'], errors="coerce")
    big_df['净利润-同比增长'] = pd.to_numeric(big_df['净利润-同比增长'], errors="coerce")
    big_df['净利润-季度环比增长'] = pd.to_numeric(big_df['净利润-季度环比增长'], errors="coerce")
    big_df['每股净资产'] = pd.to_numeric(big_df['每股净资产'], errors="coerce")
    big_df['净资产收益率'] = pd.to_numeric(big_df['净资产收益率'], errors="coerce")
    big_df['每股经营现金流量'] = pd.to_numeric(big_df['每股经营现金流量'], errors="coerce")
    big_df['销售毛利率'] = pd.to_numeric(big_df['销售毛利率'], errors="coerce")
    big_df['最新公告日期'] = pd.to_datetime(big_df['最新公告日期']).dt.date
    return big_df


if __name__ == "__main__":
    stock_yjbb_em_df = stock_yjbb_em(date="20220331")
    print(stock_yjbb_em_df)



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2023/6/19 17:00
Desc: 沐甜科技数据中心-中国食糖指数
https://www.msweet.com.cn/mtkj/sjzx13/index.html
"""
import requests
import pandas as pd


def index_sugar_msweet() -> pd.DataFrame:
    """
    沐甜科技数据中心-中国食糖指数
    https://www.msweet.com.cn/mtkj/sjzx13/index.html
    :return: 中国食糖指数
    :rtype: pandas.DataFrame
    """
    url = "https://www.msweet.com.cn/eportal/ui"
    params = {
        "struts.portlet.action": "/portlet/price!getSTZSJson.action",
        "moduleId": "cb752447cfe24b44b18c7a7e9abab048",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.concat(
        [pd.DataFrame(data_json["category"]), pd.DataFrame(data_json["data"])], axis=1
    )
    temp_df.columns = ["日期", "综合价格", "原糖价格", "现货价格"]
    temp_df.loc[3226, ["原糖价格"]] = 12.88  # 数据源错误
    temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date
    temp_df["综合价格"] = pd.to_numeric(temp_df["综合价格"], errors="coerce")
    temp_df["原糖价格"] = pd.to_numeric(temp_df["原糖价格"], errors="coerce")
    temp_df["现货价格"] = pd.to_numeric(temp_df["现货价格"], errors="coerce")
    return temp_df


def index_inner_quote_sugar_msweet() -> pd.DataFrame:
    """
    沐甜科技数据中心-配额内进口糖估算指数
    https://www.msweet.com.cn/mtkj/sjzx13/index.html
    :return: 配额内进口糖估算指数
    :rtype: pandas.DataFrame
    """
    url = "https://www.msweet.com.cn/datacenterapply/datacenter/json/JinKongTang.json"
    r = requests.get(url)
    data_json = r.json()
    temp_df = pd.concat(
        [pd.DataFrame(data_json["category"]), pd.DataFrame(data_json["data"])], axis=1
    )
    temp_df.columns = [
        "日期",
        "利润空间",
        "泰国糖",
        "泰国MA5",
        "巴西MA5",
        "利润MA5",
        "巴西MA10",
        "巴西糖",
        "柳州现货价",
        "广州现货价",
        "泰国MA10",
        "利润MA30",
        "利润MA10",
    ]
    temp_df.loc[988, ["泰国糖"]] = 4045.2  # 数据源错误
    temp_df["日期"] = temp_df["日期"].str.replace("/", "-")
    temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date
    temp_df["利润空间"] = pd.to_numeric(temp_df["利润空间"], errors="coerce")
    temp_df["泰国糖"] = pd.to_numeric(temp_df["泰国糖"], errors="coerce")
    temp_df["泰国MA5"] = pd.to_numeric(temp_df["泰国MA5"], errors="coerce")
    temp_df["巴西MA5"] = pd.to_numeric(temp_df["巴西MA5"], errors="coerce")
    temp_df["巴西MA10"] = pd.to_numeric(temp_df["巴西MA10"], errors="coerce")
    temp_df["巴西糖"] = pd.to_numeric(temp_df["巴西糖"], errors="coerce")
    temp_df["柳州现货价"] = pd.to_numeric(temp_df["柳州现货价"], errors="coerce")
    temp_df["广州现货价"] = pd.to_numeric(temp_df["广州现货价"], errors="coerce")
    temp_df["泰国MA10"] = pd.to_numeric(temp_df["泰国MA10"], errors="coerce")
    temp_df["利润MA30"] = pd.to_numeric(temp_df["利润MA30"], errors="coerce")
    temp_df["利润MA10"] = pd.to_numeric(temp_df["利润MA10"], errors="coerce")
    return temp_df


def index_outer_quote_sugar_msweet() -> pd.DataFrame:
    """
    沐甜科技数据中心-配额外进口糖估算指数
    https://www.msweet.com.cn/mtkj/sjzx13/index.html
    :return: 配额内进口糖估算指数
    :rtype: pandas.DataFrame
    """
    url = "https://www.msweet.com.cn/datacenterapply/datacenter/json/Jkpewlr.json"
    r = requests.get(url)
    data_json = r.json()
    temp_df = pd.concat(
        [pd.DataFrame(data_json["category"]), pd.DataFrame(data_json["data"])], axis=1
    )
    temp_df.columns = ["日期", "巴西糖进口成本", "泰国糖进口利润空间", "巴西糖进口利润空间", "泰国糖进口成本", "日照现货价"]
    temp_df["日期"] = temp_df["日期"].str.replace("/", "-")
    temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date
    temp_df["巴西糖进口成本"] = pd.to_numeric(temp_df["巴西糖进口成本"], errors="coerce")
    temp_df["泰国糖进口利润空间"] = pd.to_numeric(temp_df["泰国糖进口利润空间"], errors="coerce")
    temp_df["巴西糖进口利润空间"] = pd.to_numeric(temp_df["巴西糖进口利润空间"], errors="coerce")
    temp_df["泰国糖进口成本"] = pd.to_numeric(temp_df["泰国糖进口成本"])
    temp_df["日照现货价"] = pd.to_numeric(temp_df["日照现货价"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    index_sugar_msweet_df = index_sugar_msweet()
    print(index_sugar_msweet_df)

    index_inner_quote_sugar_msweet_df = index_inner_quote_sugar_msweet()
    print(index_inner_quote_sugar_msweet_df)

    index_outer_quote_sugar_msweet_df = index_outer_quote_sugar_msweet()
    print(index_outer_quote_sugar_msweet_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/5/19 18:30
Desc: 东方财富网-数据中心-特色数据-高管持股
https://data.eastmoney.com/executive/list.html
"""

import pandas as pd
import requests
from tqdm import tqdm


def stock_hold_management_detail_em() -> pd.DataFrame:
    """
    东方财富网-数据中心-特色数据-高管持股-董监高及相关人员持股变动明细
    https://data.eastmoney.com/executive/list.html
    :return: 董监高及相关人员持股变动明细
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "reportName": "RPT_EXECUTIVE_HOLD_DETAILS",
        "columns": "ALL",
        "quoteColumns": "",
        "filter": "",
        "pageNumber": "1",
        "pageSize": "5000",
        "sortTypes": "-1,1,1",
        "sortColumns": "CHANGE_DATE,SECURITY_CODE,PERSON_NAME",
        "source": "WEB",
        "client": "WEB",
        "p": "1",
        "pageNo": "1",
        "pageNum": "1",
        "_": "1691501763413",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    total_page = data_json["result"]["pages"]
    big_df = pd.DataFrame()
    for page in tqdm(range(1, total_page + 1), leave=False):
        params.update(
            {
                "pageNumber": page,
                "p": page,
                "pageNo": page,
                "pageNum": page,
            }
        )
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)

    big_df.rename(
        columns={
            "SECURITY_CODE": "代码",
            "DERIVE_SECURITY_CODE": "-",
            "SECURITY_NAME": "名称",
            "CHANGE_DATE": "日期",
            "PERSON_NAME": "变动人",
            "CHANGE_SHARES": "变动股数",
            "AVERAGE_PRICE": "成交均价",
            "CHANGE_AMOUNT": "变动金额",
            "CHANGE_REASON": "变动原因",
            "CHANGE_RATIO": "变动比例",
            "CHANGE_AFTER_HOLDNUM": "变动后持股数",
            "HOLD_TYPE": "持股种类",
            "DSE_PERSON_NAME": "董监高人员姓名",
            "POSITION_NAME": "职务",
            "PERSON_DSE_RELATION": "变动人与董监高的关系",
            "ORG_CODE": "-",
            "GGEID": "-",
            "BEGIN_HOLD_NUM": "开始时持有",
            "END_HOLD_NUM": "结束后持有",
        },
        inplace=True,
    )

    big_df = big_df[
        [
            "日期",
            "代码",
            "名称",
            "变动人",
            "变动股数",
            "成交均价",
            "变动金额",
            "变动原因",
            "变动比例",
            "变动后持股数",
            "持股种类",
            "董监高人员姓名",
            "职务",
            "变动人与董监高的关系",
            "开始时持有",
            "结束后持有",
        ]
    ]
    big_df["日期"] = pd.to_datetime(big_df["日期"], errors="coerce").dt.date
    big_df["变动股数"] = pd.to_numeric(big_df["变动股数"], errors="coerce")
    big_df["成交均价"] = pd.to_numeric(big_df["成交均价"], errors="coerce")
    big_df["变动金额"] = pd.to_numeric(big_df["变动金额"], errors="coerce")
    big_df["变动比例"] = pd.to_numeric(big_df["变动比例"], errors="coerce")
    big_df["变动后持股数"] = pd.to_numeric(big_df["变动后持股数"], errors="coerce")
    big_df["开始时持有"] = pd.to_numeric(big_df["开始时持有"], errors="coerce")
    big_df["结束后持有"] = pd.to_numeric(big_df["结束后持有"], errors="coerce")
    return big_df


def stock_hold_management_person_em(
    symbol: str = "001308", name: str = "吴远"
) -> pd.DataFrame:
    """
    东方财富网-数据中心-特色数据-高管持股-人员增减持股变动明细
    https://data.eastmoney.com/executive/personinfo.html?name=%E5%90%B4%E8%BF%9C&code=001308
    :param symbol: 股票代码
    :type name: str
    :param name: 高管名称
    :type symbol: str
    :return: 人员增减持股变动明细
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "reportName": "RPT_EXECUTIVE_HOLD_DETAILS",
        "columns": "ALL",
        "quoteColumns": "",
        "filter": f'(SECURITY_CODE="{symbol}")(PERSON_NAME="{name}")',
        "pageNumber": "1",
        "pageSize": "5000",
        "sortTypes": "-1,1,1",
        "sortColumns": "CHANGE_DATE,SECURITY_CODE,PERSON_NAME",
        "source": "WEB",
        "client": "WEB",
        "_": "1691503078611",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_df.rename(
        columns={
            "SECURITY_CODE": "代码",
            "DERIVE_SECURITY_CODE": "-",
            "SECURITY_NAME": "名称",
            "CHANGE_DATE": "日期",
            "PERSON_NAME": "变动人",
            "CHANGE_SHARES": "变动股数",
            "AVERAGE_PRICE": "成交均价",
            "CHANGE_AMOUNT": "变动金额",
            "CHANGE_REASON": "变动原因",
            "CHANGE_RATIO": "变动比例",
            "CHANGE_AFTER_HOLDNUM": "变动后持股数",
            "HOLD_TYPE": "持股种类",
            "DSE_PERSON_NAME": "董监高人员姓名",
            "POSITION_NAME": "职务",
            "PERSON_DSE_RELATION": "变动人与董监高的关系",
            "ORG_CODE": "-",
            "GGEID": "-",
            "BEGIN_HOLD_NUM": "开始时持有",
            "END_HOLD_NUM": "结束后持有",
        },
        inplace=True,
    )

    temp_df = temp_df[
        [
            "日期",
            "代码",
            "名称",
            "变动人",
            "变动股数",
            "成交均价",
            "变动金额",
            "变动原因",
            "变动比例",
            "变动后持股数",
            "持股种类",
            "董监高人员姓名",
            "职务",
            "变动人与董监高的关系",
            "开始时持有",
            "结束后持有",
        ]
    ]
    temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date
    temp_df["变动股数"] = pd.to_numeric(temp_df["变动股数"], errors="coerce")
    temp_df["成交均价"] = pd.to_numeric(temp_df["成交均价"], errors="coerce")
    temp_df["变动金额"] = pd.to_numeric(temp_df["变动金额"], errors="coerce")
    temp_df["变动比例"] = pd.to_numeric(temp_df["变动比例"], errors="coerce")
    temp_df["变动后持股数"] = pd.to_numeric(temp_df["变动后持股数"], errors="coerce")
    temp_df["开始时持有"] = pd.to_numeric(temp_df["开始时持有"], errors="coerce")
    temp_df["结束后持有"] = pd.to_numeric(temp_df["结束后持有"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    stock_hold_management_detail_em_df = stock_hold_management_detail_em()
    print(stock_hold_management_detail_em_df)

    stock_hold_management_person_em_df = stock_hold_management_person_em(
        symbol="001308", name="吴远"
    )
    print(stock_hold_management_person_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/4/29 15:00
Desc: 东方财富网-数据中心-特色数据-停复牌信息
https://data.eastmoney.com/tfpxx/
"""

import pandas as pd
import requests


def stock_tfp_em(date: str = "20240426") -> pd.DataFrame:
    """
    东方财富网-数据中心-特色数据-停复牌信息
    https://data.eastmoney.com/tfpxx/
    :param date: specific date as "2020-03-19"
    :type date: str
    :return: 停复牌信息表
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "SUSPEND_START_DATE",
        "sortTypes": "-1",
        "pageSize": "500",
        "pageNumber": "1",
        "reportName": "RPT_CUSTOM_SUSPEND_DATA_INTERFACE",
        "columns": "ALL",
        "source": "WEB",
        "client": "WEB",
        "filter": f"""(MARKET="全部")(DATETIME='{"-".join([date[:4], date[4:6], date[6:]])}')""",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    total_page = data_json["result"]["pages"]
    big_df = pd.DataFrame()
    for page in range(1, total_page + 1):
        params.update({"pageNumber": page})
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)

    big_df.reset_index(inplace=True)
    big_df.columns = [
        "序号",
        "代码",
        "名称",
        "停牌时间",
        "停牌截止时间",
        "停牌期限",
        "停牌原因",
        "所属市场",
        "停牌开始日期",
        "预计复牌时间",
        "-",
        "-",
    ]
    big_df = big_df[
        [
            "序号",
            "代码",
            "名称",
            "停牌时间",
            "停牌截止时间",
            "停牌期限",
            "停牌原因",
            "所属市场",
            "预计复牌时间",
        ]
    ]
    big_df["停牌时间"] = pd.to_datetime(big_df["停牌时间"], errors="coerce").dt.date
    big_df["停牌截止时间"] = pd.to_datetime(
        big_df["停牌截止时间"], errors="coerce"
    ).dt.date
    big_df["预计复牌时间"] = pd.to_datetime(
        big_df["预计复牌时间"], errors="coerce"
    ).dt.date
    return big_df


if __name__ == "__main__":
    stock_tfp_em_df = stock_tfp_em(date="20240426")
    print(stock_tfp_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/9/26 18:00
Desc: Drewry 集装箱指数
https://www.drewry.co.uk/supply-chain-advisors/supply-chain-expertise/world-container-index-assessed-by-drewry
https://infogram.com/world-container-index-1h17493095xl4zj
"""

import pandas as pd
import requests
from bs4 import BeautifulSoup

from byteapp_ak.utils import demjson


def drewry_wci_index(symbol: str = "composite") -> pd.DataFrame:
    """
    Drewry 集装箱指数
    https://infogram.com/world-container-index-1h17493095xl4zj
    :param symbol: choice of {"composite", "shanghai-rotterdam", "rotterdam-shanghai", "shanghai-los angeles",
    "los angeles-shanghai", "shanghai-genoa", "new york-rotterdam", "rotterdam-new york"}
    :type symbol: str
    :return: Drewry 集装箱指数
    :rtype: pandas.DataFrame
    """
    symbol_map = {
        "composite": 0,
        "shanghai-rotterdam": 1,
        "rotterdam-shanghai": 2,
        "shanghai-los angeles": 3,
        "los angeles-shanghai": 4,
        "shanghai-genoa": 5,
        "new york-rotterdam": 6,
        "rotterdam-new york": 7,
    }
    url = "https://infogram.com/world-container-index-1h17493095xl4zj"
    r = requests.get(url)
    soup = BeautifulSoup(r.text, features="lxml")
    data_text = soup.find_all("script")[-4].string.strip("window.infographicData=")[:-1]
    data_json = demjson.decode(data_text)
    data_json_need = data_json["elements"]["content"]["content"]["entities"][
        "7a55585f-3fb3-44e6-9b54-beea1cd20b4d"
    ]["data"][symbol_map[symbol]]
    date_list = [item[0]["value"] for item in data_json_need[1:]]
    try:
        value_list = [item[1]["value"] for item in data_json_need[1:]]
    except TypeError:
        value_list = [item[1]["value"] for item in data_json_need[1:-1]]
    temp_df = pd.DataFrame([date_list, value_list]).T
    temp_df.columns = ["date", "wci"]
    temp_df["date"] = pd.to_datetime(
        temp_df["date"], format="%d-%b-%y", errors="coerce"
    ).dt.date
    temp_df["wci"] = pd.to_numeric(temp_df["wci"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    drewry_wci_index_df = drewry_wci_index(symbol="composite")
    print(drewry_wci_index_df)

    drewry_wci_index_df = drewry_wci_index(symbol="shanghai-rotterdam")
    print(drewry_wci_index_df)

    drewry_wci_index_df = drewry_wci_index(symbol="rotterdam-shanghai")
    print(drewry_wci_index_df)

    drewry_wci_index_df = drewry_wci_index(symbol="shanghai-los angeles")
    print(drewry_wci_index_df)

    drewry_wci_index_df = drewry_wci_index(symbol="los angeles-shanghai")
    print(drewry_wci_index_df)

    drewry_wci_index_df = drewry_wci_index(symbol="shanghai-genoa")
    print(drewry_wci_index_df)

    drewry_wci_index_df = drewry_wci_index(symbol="new york-rotterdam")
    print(drewry_wci_index_df)

    drewry_wci_index_df = drewry_wci_index(symbol="rotterdam-new york")
    print(drewry_wci_index_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2023/12/7 15:30
Desc: 新浪财经-美股实时行情数据和历史行情数据
https://finance.sina.com.cn/stock/usstock/sector.shtml
"""

import json
from functools import lru_cache

import pandas as pd
import requests
import py_mini_racer
from tqdm import tqdm

from byteapp_ak.stock.cons import (
    js_hash_text,
    zh_js_decode,
    us_sina_stock_list_url,
    us_sina_stock_dict_payload,
    us_sina_stock_hist_qfq_url,
)


@lru_cache()
def __get_us_page_count() -> int:
    """
    新浪财经-美股-总页数
    https://finance.sina.com.cn/stock/usstock/sector.shtml
    :return: 美股总页数
    :rtype: int
    """
    page = "1"
    us_js_decode = (
        f"US_CategoryService.getList?page={page}&num=20&sort=&asc=0&market=&id="
    )
    js_code = py_mini_racer.MiniRacer()
    js_code.eval(js_hash_text)
    dict_list = js_code.call("d", us_js_decode)  # 执行js解密代码
    us_sina_stock_dict_payload.update({"page": "{}".format(page)})
    res = requests.get(
        us_sina_stock_list_url.format(dict_list),
        params=us_sina_stock_dict_payload,
    )
    data_json = json.loads(res.text[res.text.find("({") + 1 : res.text.rfind(");")])
    if not isinstance(int(data_json["count"]) / 20, int):
        page_count = int(int(data_json["count"]) / 20) + 1
    else:
        page_count = int(int(data_json["count"]) / 20)
    return page_count


@lru_cache()
def get_us_stock_name() -> pd.DataFrame:
    """
    u.s. stock's english name, chinese name and symbol
    you should use symbol to get apply into the next function
    https://finance.sina.com.cn/stock/usstock/sector.shtml
    :return: stock's english name, chinese name and symbol
    :rtype: pandas.DataFrame
    """
    big_df = pd.DataFrame()
    page_count = __get_us_page_count()
    for page in tqdm(range(1, page_count + 1), leave=False):
        us_js_decode = (
            "US_CategoryService.getList?page={}&num=20&sort=&asc=0&market=&id=".format(
                page
            )
        )
        js_code = py_mini_racer.MiniRacer()
        js_code.eval(js_hash_text)
        dict_list = js_code.call("d", us_js_decode)  # 执行js解密代码
        us_sina_stock_dict_payload.update({"page": "{}".format(page)})
        res = requests.get(
            us_sina_stock_list_url.format(dict_list),
            params=us_sina_stock_dict_payload,
        )
        data_json = json.loads(res.text[res.text.find("({") + 1 : res.text.rfind(");")])
        big_df = pd.concat([big_df, pd.DataFrame(data_json["data"])], ignore_index=True)
    return big_df[["name", "cname", "symbol"]]


def stock_us_spot() -> pd.DataFrame:
    """
    新浪财经-所有美股的数据, 注意延迟 15 分钟
    https://finance.sina.com.cn/stock/usstock/sector.shtml
    :return: 美股所有股票实时行情
    :rtype: pandas.DataFrame
    """
    big_df = pd.DataFrame()
    page_count = __get_us_page_count()
    for page in tqdm(range(1, page_count + 1), leave=False):
        # page = "1"
        us_js_decode = (
            "US_CategoryService.getList?page={}&num=20&sort=&asc=0&market=&id=".format(
                page
            )
        )
        js_code = py_mini_racer.MiniRacer()
        js_code.eval(js_hash_text)
        dict_list = js_code.call("d", us_js_decode)  # 执行js解密代码
        us_sina_stock_dict_payload.update({"page": "{}".format(page)})
        res = requests.get(
            us_sina_stock_list_url.format(dict_list),
            params=us_sina_stock_dict_payload,
        )
        data_json = json.loads(res.text[res.text.find("({") + 1 : res.text.rfind(");")])
        big_df = pd.concat([big_df, pd.DataFrame(data_json["data"])], ignore_index=True)
    return big_df


def stock_us_daily(symbol: str = "FB", adjust: str = "") -> pd.DataFrame:
    """
    新浪财经-美股
    https://finance.sina.com.cn/stock/usstock/sector.shtml
    备注：
    1. CIEN 新浪复权因子错误
    2. AI 新浪复权因子错误, 该股票刚上市未发生复权, 但是返回复权因子
    :param symbol: 可以使用 get_us_stock_name 获取
    :type symbol: str
    :param adjust: "": 返回未复权的数据 ; qfq: 返回前复权后的数据; qfq-factor: 返回前复权因子和调整;
    :type adjust: str
    :return: 指定 adjust 的数据
    :rtype: pandas.DataFrame
    """
    url = f"https://finance.sina.com.cn/staticdata/us/{symbol}"
    res = requests.get(url)
    js_code = py_mini_racer.MiniRacer()
    js_code.eval(zh_js_decode)
    dict_list = js_code.call("d", res.text.split("=")[1].split(";")[0].replace('"', ""))
    data_df = pd.DataFrame(dict_list)
    data_df["date"] = pd.to_datetime(data_df["date"]).dt.date
    data_df.index = pd.to_datetime(data_df["date"])
    del data_df["amount"]
    del data_df["date"]
    data_df = data_df.astype("float")
    url = us_sina_stock_hist_qfq_url.format(symbol)
    res = requests.get(url)
    qfq_factor_df = pd.DataFrame(eval(res.text.split("=")[1].split("\n")[0])["data"])
    qfq_factor_df.rename(
        columns={
            "c": "adjust",
            "d": "date",
            "f": "qfq_factor",
        },
        inplace=True,
    )
    qfq_factor_df.index = pd.to_datetime(qfq_factor_df["date"])
    del qfq_factor_df["date"]

    # 处理复权因子
    temp_date_range = pd.date_range("1900-01-01", qfq_factor_df.index[0].isoformat())
    temp_df = pd.DataFrame(range(len(temp_date_range)), temp_date_range)
    new_range = pd.merge(
        temp_df, qfq_factor_df, left_index=True, right_index=True, how="left"
    )
    new_range = new_range.ffill()
    new_range = new_range.iloc[:, [1, 2]]

    if adjust == "qfq":
        if len(new_range) == 1:
            new_range.index.values[0] = pd.to_datetime(str(data_df.index.date[0]))
        temp_df = pd.merge(
            data_df, new_range, left_index=True, right_index=True, how="left"
        )
        try:
            # try for pandas >= 2.1.0
            temp_df.ffill(inplace=True)
        except Exception:
            try:
                # try for pandas < 2.1.0
                temp_df.fillna(method="ffill", inplace=True)
            except Exception as e:
                print("Error:", e)
        try:
            # try for pandas >= 2.1.0
            temp_df.bfill(inplace=True)
        except Exception:
            try:
                # try for pandas < 2.1.0
                temp_df.fillna(method="bfill", inplace=True)
            except Exception as e:
                print("Error:", e)

        temp_df = temp_df.astype(float)
        temp_df["open"] = temp_df["open"] * temp_df["qfq_factor"] + temp_df["adjust"]
        temp_df["high"] = temp_df["high"] * temp_df["qfq_factor"] + temp_df["adjust"]
        temp_df["close"] = temp_df["close"] * temp_df["qfq_factor"] + temp_df["adjust"]
        temp_df["low"] = temp_df["low"] * temp_df["qfq_factor"] + temp_df["adjust"]
        temp_df = temp_df.apply(lambda x: round(x, 4))
        temp_df = temp_df.astype("float")
        # 处理复权因子错误的情况-开始
        check_df = temp_df[["open", "high", "low", "close"]].copy()
        check_df.dropna(inplace=True)
        if check_df.empty:
            data_df.reset_index(inplace=True)
            return data_df
        # 处理复权因子错误的情况-结束
        result_data = temp_df.iloc[:, :-2]
        result_data.reset_index(inplace=True)
        return result_data

    if adjust == "qfq-factor":
        qfq_factor_df.reset_index(inplace=True)
        return qfq_factor_df

    if adjust == "":
        data_df.reset_index(inplace=True)
        return data_df


if __name__ == "__main__":
    stock_us_stock_name_df = get_us_stock_name()
    print(stock_us_stock_name_df)

    stock_us_spot_df = stock_us_spot()
    print(stock_us_spot_df)

    stock_us_daily_df = stock_us_daily(symbol=".DJI", adjust="")
    print(stock_us_daily_df)

    stock_us_daily_qfq_df = stock_us_daily(symbol=".DJI", adjust="qfq")
    print(stock_us_daily_qfq_df)

    stock_us_daily_qfq_factor_df = stock_us_daily(symbol="AAPL", adjust="qfq-factor")
    print(stock_us_daily_qfq_factor_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/6/25 15:00
Desc: 碳排放交易
北京市碳排放权电子交易平台-北京市碳排放权公开交易行情
https://www.bjets.com.cn/article/jyxx/

深圳碳排放交易所-国内碳情
http://www.cerx.cn/dailynewsCN/index.htm

深圳碳排放交易所-国际碳情
http://www.cerx.cn/dailynewsOuter/index.htm

湖北碳排放权交易中心-现货交易数据-配额-每日概况
http://www.cerx.cn/dailynewsOuter/index.htm

广州碳排放权交易中心-行情信息
http://www.cnemission.com/article/hqxx/
"""

from io import StringIO

import pandas as pd
import requests
from bs4 import BeautifulSoup
from tqdm import tqdm

from byteapp_ak.utils import demjson
from byteapp_ak.utils.cons import headers


def energy_carbon_domestic(symbol: str = "湖北") -> pd.DataFrame:
    """
    碳交易网-行情信息
    http://www.tanjiaoyi.com/
    :param symbol: choice of {'湖北', '上海', '北京', '重庆', '广东', '天津', '深圳', '福建'}
    :type symbol: str
    :return: 行情信息
    :rtype: pandas.DataFrame
    """
    url = "http://k.tanjiaoyi.com:8080/KDataController/getHouseDatasInAverage.do"
    params = {
        "lcnK": "53f75bfcefff58e4046ccfa42171636c",
        "brand": "TAN",
        "_": "1626773022063",
    }
    r = requests.get(url, params=params)
    data_text = r.text
    data_json = demjson.decode(data_text[data_text.find("(") + 1 : -1])
    temp_df = pd.DataFrame(data_json[symbol])
    temp_df.columns = [
        "成交价",
        "_",
        "成交量",
        "地点",
        "成交额",
        "日期",
        "_",
    ]
    temp_df = temp_df[
        [
            "日期",
            "成交价",
            "成交量",
            "成交额",
            "地点",
        ]
    ]
    temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date
    temp_df["成交价"] = pd.to_numeric(temp_df["成交价"], errors="coerce")
    temp_df["成交量"] = pd.to_numeric(temp_df["成交量"], errors="coerce")
    temp_df["成交额"] = pd.to_numeric(temp_df["成交额"], errors="coerce")
    return temp_df


def energy_carbon_bj() -> pd.DataFrame:
    """
    北京市碳排放权电子交易平台-北京市碳排放权公开交易行情
    https://www.bjets.com.cn/article/jyxx/
    :return: 北京市碳排放权公开交易行情
    :rtype: pandas.DataFrame
    """
    url = "https://www.bjets.com.cn/article/jyxx/"
    r = requests.get(url, verify=False, headers=headers)
    soup = BeautifulSoup(r.text, features="lxml")
    total_page = (
        soup.find("table")
        .find("script")
        .string.split("=")[-1]
        .strip()
        .strip(";")
        .strip('"')
    )
    temp_df = pd.DataFrame()
    for i in tqdm(
        range(1, int(total_page) + 1),
        desc="Please wait for a moment",
        leave=False,
    ):
        if i == 1:
            i = ""
        url = f"https://www.bjets.com.cn/article/jyxx/?{i}"
        r = requests.get(url, verify=False, headers=headers)
        r.encoding = "utf-8"
        df = pd.read_html(StringIO(r.text))[0]
        temp_df = pd.concat(objs=[temp_df, df], ignore_index=True)
    temp_df.columns = ["日期", "成交量", "成交均价", "成交额"]
    temp_df["成交单位"] = (
        temp_df["成交额"]
        .str.split("(", expand=True)
        .iloc[:, 1]
        .str.split("）", expand=True)
        .iloc[:, 0]
        .str.split(")", expand=True)
        .iloc[:, 0]
    )
    temp_df["成交额"] = (
        temp_df["成交额"]
        .str.split("(", expand=True)
        .iloc[:, 0]
        .str.split("（", expand=True)
        .iloc[:, 0]
    )
    temp_df["成交量"] = pd.to_numeric(temp_df["成交量"], errors="coerce")
    temp_df["成交均价"] = pd.to_numeric(temp_df["成交均价"], errors="coerce")
    temp_df["成交额"] = temp_df["成交额"].str.replace(",", "")
    temp_df["成交额"] = pd.to_numeric(temp_df["成交额"], errors="coerce")
    temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date
    temp_df.sort_values(by="日期", inplace=True)
    temp_df.reset_index(inplace=True, drop=True)
    return temp_df


def energy_carbon_sz() -> pd.DataFrame:
    """
    深圳碳排放交易所-国内碳情
    http://www.cerx.cn/dailynewsCN/index.htm
    :return: 国内碳情每日行情数据
    :rtype: pandas.DataFrame
    """
    url = "http://www.cerx.cn/dailynewsCN/index.htm"
    r = requests.get(url, headers=headers)
    soup = BeautifulSoup(r.text, features="lxml")
    page_num = int(soup.find(attrs={"class": "pagebar"}).find_all("option")[-1].text)
    big_df = pd.read_html(StringIO(r.text), header=0)[0]
    for page in tqdm(
        range(2, page_num + 1), desc="Please wait for a moment", leave=False
    ):
        url = f"http://www.cerx.cn/dailynewsCN/index_{page}.htm"
        r = requests.get(url, headers=headers)
        temp_df = pd.read_html(StringIO(r.text), header=0)[0]
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
    big_df["交易日期"] = pd.to_datetime(big_df["交易日期"], errors="coerce").dt.date
    big_df["开盘价"] = pd.to_numeric(big_df["开盘价"], errors="coerce")
    big_df["最高价"] = pd.to_numeric(big_df["最高价"], errors="coerce")
    big_df["最低价"] = pd.to_numeric(big_df["最低价"], errors="coerce")
    big_df["成交均价"] = pd.to_numeric(big_df["成交均价"], errors="coerce")
    big_df["收盘价"] = pd.to_numeric(big_df["收盘价"], errors="coerce")
    big_df["成交量"] = pd.to_numeric(big_df["成交量"], errors="coerce")
    big_df["成交额"] = pd.to_numeric(big_df["成交额"], errors="coerce")
    big_df.sort_values(by="交易日期", inplace=True)
    big_df.reset_index(inplace=True, drop=True)
    return big_df


def energy_carbon_eu() -> pd.DataFrame:
    """
    深圳碳排放交易所-国际碳情
    http://www.cerx.cn/dailynewsOuter/index.htm
    :return: 国际碳情每日行情数据
    :rtype: pandas.DataFrame
    """
    url = "http://www.cerx.cn/dailynewsOuter/index.htm"
    r = requests.get(url, headers=headers)
    soup = BeautifulSoup(r.text, features="lxml")
    page_num = int(soup.find(attrs={"class": "pagebar"}).find_all("option")[-1].text)
    big_df = pd.read_html(StringIO(r.text), header=0)[0]
    for page in tqdm(
        range(2, page_num + 1), desc="Please wait for a moment", leave=False
    ):
        url = f"http://www.cerx.cn/dailynewsOuter/index_{page}.htm"
        r = requests.get(url)
        temp_df = pd.read_html(StringIO(r.text), header=0)[0]
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
    big_df["交易日期"] = pd.to_datetime(big_df["交易日期"], errors="coerce").dt.date
    big_df["开盘价"] = pd.to_numeric(big_df["开盘价"], errors="coerce")
    big_df["最高价"] = pd.to_numeric(big_df["最高价"], errors="coerce")
    big_df["最低价"] = pd.to_numeric(big_df["最低价"], errors="coerce")
    big_df["成交均价"] = pd.to_numeric(big_df["成交均价"], errors="coerce")
    big_df["收盘价"] = pd.to_numeric(big_df["收盘价"], errors="coerce")
    big_df["成交量"] = pd.to_numeric(big_df["成交量"], errors="coerce")
    big_df["成交额"] = pd.to_numeric(big_df["成交额"], errors="coerce")
    big_df.sort_values(by="交易日期", inplace=True)
    big_df.reset_index(inplace=True, drop=True)
    return big_df


def energy_carbon_hb() -> pd.DataFrame:
    """
    湖北碳排放权交易中心-现货交易数据-配额-每日概况
    http://www.hbets.cn/list/13.html?page=42
    :return: 现货交易数据-配额-每日概况行情数据
    :rtype: pandas.DataFrame
    """
    url = "https://www.hbets.cn/"
    r = requests.get(url, headers=headers)
    soup = BeautifulSoup(r.text, features="lxml")
    data_text = (
        soup.find(name="div", attrs={"class": "threeLeft"}).find_all("script")[1].text
    )
    start_pos = data_text.find("cjj = '[") + 7  # 找到 JSON 数组开始的位置
    end_pos = data_text.rfind("cjj =") - 31  # 找到 JSON 数组结束的位置
    data_json = demjson.decode(data_text[start_pos:end_pos])
    temp_df = pd.DataFrame.from_dict(data_json)
    temp_df.rename(
        columns={
            "riqi": "日期",
            "cjj": "成交价",
            "cjl": "成交量",
            "zx": "最新",
            "zd": "涨跌",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "日期",
            "成交价",
            "成交量",
            "最新",
            "涨跌",
        ]
    ]
    temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date
    temp_df["成交价"] = pd.to_numeric(temp_df["成交价"], errors="coerce")
    temp_df["成交量"] = pd.to_numeric(temp_df["成交量"], errors="coerce")
    temp_df["最新"] = pd.to_numeric(temp_df["最新"], errors="coerce")
    temp_df["涨跌"] = pd.to_numeric(temp_df["涨跌"], errors="coerce")
    return temp_df


def energy_carbon_gz() -> pd.DataFrame:
    """
    广州碳排放权交易中心-行情信息
    http://www.cnemission.com/article/hqxx/
    :return: 行情信息数据
    :rtype: pandas.DataFrame
    """
    url = "http://ets.cnemission.com/carbon/portalIndex/markethistory"
    params = {
        "Top": "1",
        "beginTime": "2010-01-01",
        "endTime": "2030-09-12",
    }
    r = requests.get(url, params=params)
    temp_df = pd.read_html(StringIO(r.text), header=0)[1]
    temp_df.columns = [
        "日期",
        "品种",
        "开盘价",
        "收盘价",
        "最高价",
        "最低价",
        "涨跌",
        "涨跌幅",
        "成交数量",
        "成交金额",
    ]
    temp_df["日期"] = pd.to_datetime(
        temp_df["日期"], format="%Y%m%d", errors="coerce"
    ).dt.date
    temp_df["开盘价"] = pd.to_numeric(temp_df["开盘价"], errors="coerce")
    temp_df["收盘价"] = pd.to_numeric(temp_df["收盘价"], errors="coerce")
    temp_df["最高价"] = pd.to_numeric(temp_df["最高价"], errors="coerce")
    temp_df["最低价"] = pd.to_numeric(temp_df["最低价"], errors="coerce")
    temp_df["涨跌"] = pd.to_numeric(temp_df["涨跌"], errors="coerce")
    temp_df["涨跌幅"] = temp_df["涨跌幅"].str.strip("%")
    temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"], errors="coerce")
    temp_df["成交数量"] = pd.to_numeric(temp_df["成交数量"], errors="coerce")
    temp_df["成交金额"] = pd.to_numeric(temp_df["成交金额"], errors="coerce")
    temp_df.sort_values(by="日期", inplace=True)
    temp_df.reset_index(inplace=True, drop=True)
    return temp_df


if __name__ == "__main__":
    energy_carbon_domestic_df = energy_carbon_domestic(symbol="湖北")
    print(energy_carbon_domestic_df)

    energy_carbon_domestic_df = energy_carbon_domestic(symbol="深圳")
    print(energy_carbon_domestic_df)

    energy_carbon_bj_df = energy_carbon_bj()
    print(energy_carbon_bj_df)

    energy_carbon_sz_df = energy_carbon_sz()
    print(energy_carbon_sz_df)

    energy_carbon_eu_df = energy_carbon_eu()
    print(energy_carbon_eu_df)

    energy_carbon_hb_df = energy_carbon_hb()
    print(energy_carbon_hb_df)

    energy_carbon_gz_df = energy_carbon_gz()
    print(energy_carbon_gz_df)
